@@ -8,16 +8,18 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2009 by Bradford W. Mott and the Stella team
+// Copyright (c) 1995-2010 by Bradford W. Mott, Stephen Anthony
+// and the Stella Team
 //
-// See the file "license" for information on usage and redistribution of
+// See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: RamCheat.cxx 1724 2009-05-13 13:55:40Z stephena $
+// $Id: RamCheat.cxx 2155 2010-10-18 18:39:57Z stephena $
 //============================================================================
 
 #include "Console.hxx"
 #include "System.hxx"
+#include "OSystem.hxx"
 #include "CheatManager.hxx"
 
 #include "RamCheat.hxx"
