@@ -8,12 +8,13 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2009 by Bradford W. Mott and the Stella team
+// Copyright (c) 1995-2010 by Bradford W. Mott, Stephen Anthony
+// and the Stella Team
 //
-// See the file "license" for information on usage and redistribution of
+// See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: PromptWidget.cxx 1724 2009-05-13 13:55:40Z stephena $
+// $Id: PromptWidget.cxx 2183 2010-11-11 16:46:41Z stephena $
 //
 //   Based on code from ScummVM - Scumm Interpreter
 //   Copyright (C) 2002-2004 The ScummVM project
@@ -21,6 +22,7 @@
 
 #include <iostream>
 #include <fstream>
+#include <algorithm>
 
 #include "ScrollBarWidget.hxx"
 #include "FrameBuffer.hxx"
@@ -29,20 +31,13 @@
 #include "Debugger.hxx"
 #include "DebuggerDialog.hxx"
 #include "DebuggerParser.hxx"
+#include "StringList.hxx"
 
 #include "PromptWidget.hxx"
-#include "EquateList.hxx"
+#include "CartDebug.hxx"
 
 #define PROMPT  "> "
 
-/* TODO:
- * - it is very inefficient to redraw the full thingy when just one char is added/removed.
- *   Instead, we could just copy the GFX of the blank console (i.e. after the transparent
- *   background is drawn, before any text is drawn). Then using that, it becomes trivial
- *   to erase a single character, do scrolling etc.
- * - a *lot* of others things, this code is in no way complete and heavily under progress
- */
-
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 PromptWidget::PromptWidget(GuiObject* boss, const GUI::Font& font,
                            int x, int y, int w, int h)
@@ -65,14 +60,8 @@ PromptWidget::PromptWidget(GuiObject* boss, const GUI::Font& font,
   // Calculate depending values
   _lineWidth = (_w - kScrollBarWidth - 2) / _kConsoleCharWidth;
   _linesPerPage = (_h - 2) / _kConsoleLineHeight;
-
-  memset(_buffer, 0, kBufferSize * sizeof(int));
   _linesInBuffer = kBufferSize / _lineWidth;
 
-  _currentPos = 0;
-  _scrollLine = _linesPerPage - 1;
-  _firstLineInBuffer = 0;
-
   // Add scrollbar
   _scrollBar = new ScrollBarWidget(boss, font, _x + _w, _y, kScrollBarWidth, _h);
   _scrollBar->setTarget(this);
@@ -80,14 +69,7 @@ PromptWidget::PromptWidget(GuiObject* boss, const GUI::Font& font,
   // Init colors
   _inverse = false;
 
-  // Init History
-  _historyIndex = 0;
-  _historyLine = 0;
-  _historySize = 0;
-  for (int i = 0; i < kHistorySize; i++)
-    _history[i][0] = '\0';
-
-  _promptStartPos = _promptEndPos = -1;
+  clearScreen();
 
   addFocusWidget(this);
 }
@@ -197,7 +179,7 @@ bool PromptWidget::handleKeyDown(int ascii, int keycode, int modifiers)
           _exitedEarly = true;
           return true;
         }
-        else
+        else if(result != "")
           print(result + "\n");
       }
 
@@ -216,15 +198,16 @@ bool PromptWidget::handleKeyDown(int ascii, int keycode, int modifiers)
 
       scrollToCurrent();
       int len = _promptEndPos - _promptStartPos;
+      if(len > 256) len = 256;
 
       int lastDelimPos = -1;
       char delimiter = '\0';
 
-      char *str = new char[len + 1];
+      char str[256];
       for (i = 0; i < len; i++)
       {
         str[i] = buffer(_promptStartPos + i) & 0x7f;
-        if(strchr("*@<> ", str[i]) != NULL )
+        if(strchr("{*@<> ", str[i]) != NULL )
         {
           lastDelimPos = i;
           delimiter = str[i];
@@ -232,46 +215,50 @@ bool PromptWidget::handleKeyDown(int ascii, int keycode, int modifiers)
       }
       str[len] = '\0';
 
-      const char *completionList;
-      const char *prefix;
-      int possibilities;
+      StringList list;
+      string completionList;
+      string prefix;
 
       if(lastDelimPos < 0)
       {
         // no delimiters, do command completion:
-        DebuggerParser& parser = instance().debugger().parser();
-        possibilities = parser.countCompletions(str);
+        const DebuggerParser& parser = instance().debugger().parser();
+        parser.getCompletions(str, list);
 
-        if(possibilities < 1) {
-          delete[] str;
+        if(list.size() < 1)
           break;
-        }
 
-        completionList = parser.getCompletions();
-        prefix = parser.getCompletionPrefix();
+        sort(list.begin(), list.end());
+        completionList = list[0];
+        for(uInt32 i = 1; i < list.size(); ++i)
+          completionList += " " + list[i];
+        prefix = getCompletionPrefix(list, str);
       }
       else
       {
-        // we got a delimiter, so this must be a label:
-        EquateList& equates = instance().debugger().equates();
-        possibilities = equates.countCompletions(str + lastDelimPos + 1);
+        // we got a delimiter, so this must be a label or a function
+        const Debugger& dbg = instance().debugger();
 
-        if(possibilities < 1) {
-          delete[] str;
+        dbg.cartDebug().getCompletions(str + lastDelimPos + 1, list);
+        dbg.getCompletions(str + lastDelimPos + 1, list);
+
+        if(list.size() < 1)
           break;
-        }
 
-        // TODO - perhaps use strings instead of char pointers
-        completionList = equates.getCompletions().c_str();
-        prefix = equates.getCompletionPrefix().c_str();
+        sort(list.begin(), list.end());
+        completionList = list[0];
+        for(uInt32 i = 1; i < list.size(); ++i)
+          completionList += " " + list[i];
+        prefix = getCompletionPrefix(list, str + lastDelimPos + 1);
       }
 
-      if(possibilities == 1)
+      if(list.size() == 1)
       {
         // add to buffer as though user typed it (plus a space)
         _currentPos = _promptStartPos + lastDelimPos + 1;
-        while(*completionList != '\0')
-          putcharIntern(*completionList++);
+        const char* clptr = completionList.c_str();
+        while(*clptr != '\0')
+          putcharIntern(*clptr++);
 
         putcharIntern(' ');
         _promptEndPos = _currentPos;
@@ -298,7 +285,6 @@ bool PromptWidget::handleKeyDown(int ascii, int keycode, int modifiers)
         print(prefix);
         _promptEndPos = _currentPos;
       }
-      delete[] str;
       dirty = true;
       break;
     }
@@ -507,15 +493,17 @@ void PromptWidget::loadConfig()
     // Display greetings & prompt
     string version = string("Stella ") + STELLA_VERSION + "\n";
     print(version.c_str());
-    print("Debugger is ready\n");
     print(PROMPT);
-    _promptStartPos = _promptEndPos = _currentPos;
-
-    _firstTime = false;
-    _exitedEarly = false;
 
     // Take care of one-time debugger stuff
-    instance().debugger().autoExec();
+    print(instance().debugger().autoExec().c_str());
+    print(instance().debugger().cartDebug().loadConfigFile() + "\n");
+    print(instance().debugger().cartDebug().loadSymbolFile() + "\n");
+    print(PROMPT);
+
+    _promptStartPos = _promptEndPos = _currentPos;
+    _firstTime = false;
+    _exitedEarly = false;
   }
   else if(_exitedEarly)
   {
@@ -524,6 +512,12 @@ void PromptWidget::loadConfig()
   }
 }
 
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+int PromptWidget::getWidth() const
+{
+  return _w + kScrollBarWidth;
+}
+
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void PromptWidget::specialKeys(int keycode)
 {
@@ -795,7 +789,8 @@ void PromptWidget::putcharIntern(int c)
                       // OverlayColor contains 256 of them
     _textcolor = (c & 0x7f) << 1;
   }
-  else if(c < ' ') { // More colors (the regular GUI ones)
+  else if(c < 0x1e) { // first actual character is large dash
+    // More colors (the regular GUI ones)
     _textcolor = c + 0x100;
   }
   else if(c == 0x7f) { // toggle inverse video (DEL char)
@@ -849,7 +844,7 @@ void PromptWidget::scrollToCurrent()
 
   if (line + _linesPerPage <= _scrollLine)
   {
-    // TODO - this should only occur for loong edit lines, though
+    // TODO - this should only occur for long edit lines, though
   }
   else if (line > _scrollLine)
   {
@@ -884,3 +879,50 @@ bool PromptWidget::saveBuffer(string& filename)
   out.close();
   return true;
 }
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+string PromptWidget::getCompletionPrefix(const StringList& completions, string prefix)
+{
+  // Search for prefix in every string, progressively growing it
+  // Once a mismatch is found or length is past one of the strings, we're done
+  // We *could* use the longest common string algorithm, but for the lengths
+  // of the strings we're dealing with, it's probably not worth it
+  for(;;)
+  {
+    for(uInt32 i = 0; i < completions.size(); ++i)
+    {
+      const string& s = completions[i];
+      if(s.length() < prefix.length())
+        return prefix;  // current prefix is the best we're going to get
+      else if(!BSPF_startsWithIgnoreCase(s, prefix))
+      {
+        prefix.erase(prefix.length()-1);
+        return prefix;
+      }
+    }
+    if(completions[0].length() > prefix.length())
+      prefix = completions[0].substr(0, prefix.length() + 1);
+    else
+      return prefix;
+  }
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void PromptWidget::clearScreen()
+{
+  // Initialize start position and history
+  _currentPos = 0;
+  _scrollLine = _linesPerPage - 1;
+  _firstLineInBuffer = 0;
+  _promptStartPos = _promptEndPos = -1;
+  memset(_buffer, 0, kBufferSize * sizeof(int));
+
+  _historyIndex = 0;
+  _historyLine = 0;
+  _historySize = 0;
+  for (int i = 0; i < kHistorySize; i++)
+    _history[i][0] = '\0';
+
+  if(!_firstTime)
+    updateScrollBuffer();
+}
