@@ -8,12 +8,13 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2009 by Bradford W. Mott and the Stella team
+// Copyright (c) 1995-2010 by Bradford W. Mott, Stephen Anthony
+// and the Stella Team
 //
-// See the file "license" for information on usage and redistribution of
+// See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: TiaWidget.cxx 1724 2009-05-13 13:55:40Z stephena $
+// $Id: TiaWidget.cxx 2001 2010-04-10 21:37:23Z stephena $
 //
 //   Based on code from ScummVM - Scumm Interpreter
 //   Copyright (C) 2002-2004 The ScummVM project
@@ -25,6 +26,7 @@
 #include "FrameBuffer.hxx"
 #include "GuiObject.hxx"
 #include "OSystem.hxx"
+#include "CartDebug.hxx"
 #include "TIADebug.hxx"
 #include "ToggleBitWidget.hxx"
 #include "TogglePixelWidget.hxx"
@@ -598,6 +600,7 @@ void TiaWidget::handleCommand(CommandSender* sender, int cmd, int data, int id)
   string buf;
 
   Debugger& dbg = instance().debugger();
+  CartDebug& cart = dbg.cartDebug();
   TIADebug& tia = dbg.tiaDebug();
 
   switch(cmd)
@@ -751,7 +754,7 @@ void TiaWidget::handleCommand(CommandSender* sender, int cmd, int data, int id)
 
           // We're using the read-addresses here
           // Should we also add write-addresses, or remove this entirely?
-          myLabel->setEditString(dbg.equates().getLabel(addr, true));
+          myLabel->setEditString(cart.getLabel(addr, true));
 
           myDecValue->setEditString(dbg.valueToString(value, kBASE_10));
           myBinValue->setEditString(dbg.valueToString(value, kBASE_2));
