@@ -8,12 +8,13 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2009 by Bradford W. Mott and the Stella team
+// Copyright (c) 1995-2010 by Bradford W. Mott, Stephen Anthony
+// and the Stella Team
 //
-// See the file "license" for information on usage and redistribution of
+// See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: Cart.cxx 1847 2009-08-03 21:18:16Z stephena $
+// $Id: Cart.cxx 2162 2010-10-23 20:57:31Z stephena $
 //============================================================================
 
 #include <cassert>
@@ -30,20 +31,21 @@
 #include "Cart4K.hxx"
 #include "CartAR.hxx"
 #include "CartDPC.hxx"
+#include "CartDPCPlus.hxx"
 #include "CartE0.hxx"
 #include "CartE7.hxx"
 #include "CartEF.hxx"
 #include "CartEFSC.hxx"
+#include "CartF0.hxx"
 #include "CartF4.hxx"
 #include "CartF4SC.hxx"
 #include "CartF6.hxx"
 #include "CartF6SC.hxx"
 #include "CartF8.hxx"
 #include "CartF8SC.hxx"
-#include "CartFASC.hxx"
+#include "CartFA.hxx"
 #include "CartFE.hxx"
 #include "CartMC.hxx"
-#include "CartMB.hxx"
 #include "CartCV.hxx"
 #include "CartUA.hxx"
 #include "CartSB.hxx"
@@ -51,23 +53,17 @@
 #include "MD5.hxx"
 #include "Props.hxx"
 #include "Settings.hxx"
+#ifdef DEBUGGER_SUPPORT
+  #include "Debugger.hxx"
+  #include "CartDebug.hxx"
+#endif
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 Cartridge* Cartridge::create(const uInt8* image, uInt32 size, string& md5,
-    string& id, string type, Settings& settings)
+     string& dtype, string& id, Settings& settings)
 {
   Cartridge* cartridge = 0;
-
-  // First consider the ROMs that are special and don't have a properties entry
-  // Hopefully this list will be very small
-  if(md5 == "bc24440b59092559a1ec26055fd1270e" ||
-     md5 == "75ee371ccfc4f43e7d9b8f24e1266b55" ||
-     md5 == "9905f9f4706223dadee84f6867ede8e3")
-  {
-    // These two ROMs are normal 8K images, except they must be initialized
-    // from the opposite bank compared to normal ones
-    type = "F8 swapped";
-  }
+  string type = dtype;
 
   // Collect some info about the ROM
   ostringstream buf;
@@ -77,7 +73,7 @@ Cartridge* Cartridge::create(const uInt8* image, uInt32 size, string& md5,
   string autodetect = "";
   if(type == "AUTO-DETECT" || settings.getBool("rominfo"))
   {
-    string detected = autodetectType(image, size);
+    const string& detected = autodetectType(image, size);
     autodetect = "*";
     if(type != "AUTO-DETECT" && type != detected)
       cerr << "Auto-detection not consistent: " << type << ", " << detected << endl;
@@ -92,6 +88,7 @@ Cartridge* Cartridge::create(const uInt8* image, uInt32 size, string& md5,
     // Make sure we have a valid sized image
     if(size == 2*2048 || size == 2*4096 || size == 2*8192 || size == 2*16384)
     {
+      dtype = type;
       type = createFromMultiCart(image, size, 2, md5, id, settings);
       buf << id;
     }
@@ -102,6 +99,7 @@ Cartridge* Cartridge::create(const uInt8* image, uInt32 size, string& md5,
     // Make sure we have a valid sized image
     if(size == 4*2048 || size == 4*4096 || size == 4*8192)
     {
+      dtype = type;
       type = createFromMultiCart(image, size, 4, md5, id, settings);
       buf << id;
     }
@@ -111,15 +109,27 @@ Cartridge* Cartridge::create(const uInt8* image, uInt32 size, string& md5,
     // Make sure we have a valid sized image
     if(size == 8*2048 || size == 8*4096 || size == 8*8192)
     {
+      dtype = type;
       type = createFromMultiCart(image, size, 8, md5, id, settings);
       buf << id;
     }
   }
+  else if(type == "16IN1")
+  {
+    // Make sure we have a valid sized image
+    if(size == 16*2048 || size == 16*4096 || size == 16*8192)
+    {
+      dtype = type;
+      type = createFromMultiCart(image, size, 16, md5, id, settings);
+      buf << id;
+    }
+  }
   else if(type == "32IN1")
   {
     // Make sure we have a valid sized image
     if(size == 32*2048 || size == 32*4096)
     {
+      dtype = type;
       type = createFromMultiCart(image, size, 32, md5, id, settings);
       buf << id;
     }
@@ -127,59 +137,59 @@ Cartridge* Cartridge::create(const uInt8* image, uInt32 size, string& md5,
 
   // We should know the cart's type by now so let's create it
   if(type == "2K")
-    cartridge = new Cartridge2K(image, size);
+    cartridge = new Cartridge2K(image, size, settings);
   else if(type == "3E")
-    cartridge = new Cartridge3E(image, size);
+    cartridge = new Cartridge3E(image, size, settings);
   else if(type == "3F")
-    cartridge = new Cartridge3F(image, size);
+    cartridge = new Cartridge3F(image, size, settings);
   else if(type == "4A50")
-    cartridge = new Cartridge4A50(image, size);
+    cartridge = new Cartridge4A50(image, size, settings);
   else if(type == "4K")
-    cartridge = new Cartridge4K(image);
+    cartridge = new Cartridge4K(image, settings);
   else if(type == "AR")
     cartridge = new CartridgeAR(image, size, settings);
   else if(type == "DPC")
-    cartridge = new CartridgeDPC(image, size);
+    cartridge = new CartridgeDPC(image, size, settings);
+  else if(type == "DPC+")
+    cartridge = new CartridgeDPCPlus(image, size, settings);
   else if(type == "E0")
-    cartridge = new CartridgeE0(image);
+    cartridge = new CartridgeE0(image, settings);
   else if(type == "E7")
-    cartridge = new CartridgeE7(image);
+    cartridge = new CartridgeE7(image, settings);
   else if(type == "EF")
-    cartridge = new CartridgeEF(image);
+    cartridge = new CartridgeEF(image, settings);
   else if(type == "EFSC")
-    cartridge = new CartridgeEFSC(image);
+    cartridge = new CartridgeEFSC(image, settings);
   else if(type == "F4")
-    cartridge = new CartridgeF4(image);
+    cartridge = new CartridgeF4(image, settings);
   else if(type == "F4SC")
-    cartridge = new CartridgeF4SC(image);
+    cartridge = new CartridgeF4SC(image, settings);
   else if(type == "F6")
-    cartridge = new CartridgeF6(image);
+    cartridge = new CartridgeF6(image, settings);
   else if(type == "F6SC")
-    cartridge = new CartridgeF6SC(image);
+    cartridge = new CartridgeF6SC(image, settings);
   else if(type == "F8")
-    cartridge = new CartridgeF8(image);
-  else if(type == "F8 swapped")
-    cartridge = new CartridgeF8(image, true);
+    cartridge = new CartridgeF8(image, md5, settings);
   else if(type == "F8SC")
-    cartridge = new CartridgeF8SC(image);
+    cartridge = new CartridgeF8SC(image, settings);
   else if(type == "FA" || type == "FASC")
-    cartridge = new CartridgeFASC(image);
+    cartridge = new CartridgeFA(image, settings);
   else if(type == "FE")
-    cartridge = new CartridgeFE(image);
+    cartridge = new CartridgeFE(image, settings);
   else if(type == "MC")
-    cartridge = new CartridgeMC(image, size);
+    cartridge = new CartridgeMC(image, size, settings);
   else if(type == "F0" || type == "MB")
-    cartridge = new CartridgeMB(image);
+    cartridge = new CartridgeF0(image, settings);
   else if(type == "CV")
-    cartridge = new CartridgeCV(image, size);
+    cartridge = new CartridgeCV(image, size, settings);
   else if(type == "UA")
-    cartridge = new CartridgeUA(image);
+    cartridge = new CartridgeUA(image, settings);
   else if(type == "0840")
-    cartridge = new Cartridge0840(image);
+    cartridge = new Cartridge0840(image, settings);
   else if(type == "SB")
-    cartridge = new CartridgeSB(image, size);
+    cartridge = new CartridgeSB(image, size, settings);
   else if(type == "X07")
-    cartridge = new CartridgeX07(image);
+    cartridge = new CartridgeX07(image, settings);
   else
     cerr << "ERROR: Invalid cartridge type " << type << " ..." << endl;
 
@@ -202,7 +212,7 @@ string Cartridge::createFromMultiCart(const uInt8*& image, uInt32& size,
   image += i*size;
 
   // We need a new md5 and name
-  md5  = MD5(image, size);
+  md5 = MD5(image, size);
   ostringstream buf;
   buf << " [G" << (i+1) << "]";
   id = buf.str();
@@ -217,14 +227,20 @@ string Cartridge::createFromMultiCart(const uInt8*& image, uInt32& size,
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-Cartridge::Cartridge()
+Cartridge::Cartridge(const Settings& settings)
+  : mySettings(settings),
+    myStartBank(0),
+    myBankChanged(true),
+    myCodeAccessBase(NULL),
+    myBankLocked(false)
 {
-  unlockBank();
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 Cartridge::~Cartridge()
 {
+  if(myCodeAccessBase)
+    delete[] myCodeAccessBase;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -232,7 +248,7 @@ bool Cartridge::save(ofstream& out)
 {
   int size = -1;
 
-  uInt8* image = getImage(size);
+  const uInt8* image = getImage(size);
   if(image == 0 || size <= 0)
   {
     cerr << "save not supported" << endl;
@@ -245,16 +261,51 @@ bool Cartridge::save(ofstream& out)
   return true;
 }
 
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+uInt16 Cartridge::startBank()
+{
+  return myStartBank;
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+bool Cartridge::bankChanged()
+{
+  bool changed = myBankChanged;
+  myBankChanged = false;
+  return changed;
+}
+
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void Cartridge::registerRamArea(uInt16 start, uInt16 size,
                                 uInt16 roffset, uInt16 woffset)
 {
+#ifdef DEBUGGER_SUPPORT
   RamArea area;
   area.start   = start;
   area.size    = size;
   area.roffset = roffset;
   area.woffset = woffset;
   myRamAreaList.push_back(area);
+#endif
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void Cartridge::triggerReadFromWritePort(uInt16 address)
+{
+#ifdef DEBUGGER_SUPPORT
+  Debugger::debugger().cartDebug().triggerReadFromWritePort(address);
+#endif
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void Cartridge::createCodeAccessBase(uInt32 size)
+{
+#ifdef DEBUGGER_SUPPORT
+  myCodeAccessBase = new uInt8[size];
+  memset(myCodeAccessBase, CartDebug::ROW, size);
+#else
+  myCodeAccessBase = NULL;
+#endif
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -286,7 +337,7 @@ string Cartridge::autodetectType(const uInt8* image, uInt32 size)
     else
       type = "4K";
   }
-  else if(size == 8192)  // 8K
+  else if(size == 8*1024)  // 8K
   {
     if(isProbablySC(image, size))
       type = "F8SC";
@@ -307,19 +358,15 @@ string Cartridge::autodetectType(const uInt8* image, uInt32 size)
     else
       type = "F8";
   }
-  else if((size == 10495) || (size == 10496) || (size == 10240))  // 10K - Pitfall2
+  else if(size >= 10240 && size <= 10496)  // ~10K - Pitfall2
   {
     type = "DPC";
   }
-  else if(size == 12288)  // 12K
+  else if(size == 12*1024)  // 12K
   {
-    // TODO - this should really be in a method that checks the first
-    // 512 bytes of ROM and finds if either the lower 256 bytes or
-    // higher 256 bytes are all the same.  For now, we assume that
-    // all carts of 12K are CBS RAM Plus/FASC.
     type = "FA";
   }
-  else if(size == 16384)  // 16K
+  else if(size == 16*1024)  // 16K
   {
     if(isProbablySC(image, size))
       type = "F6SC";
@@ -332,7 +379,11 @@ string Cartridge::autodetectType(const uInt8* image, uInt32 size)
     else
       type = "F6";
   }
-  else if(size == 32768)  // 32K
+  else if(size == 29*1024)  // 29K
+  {
+    type = "DPC+";
+  }
+  else if(size == 32*1024)  // 32K
   {
     if(isProbablySC(image, size))
       type = "F4SC";
@@ -340,10 +391,12 @@ string Cartridge::autodetectType(const uInt8* image, uInt32 size)
       type = "3E";
     else if(isProbably3F(image, size))
       type = "3F";
+    else if(isProbablyDPCplus(image, size))
+      type = "DPC+";
     else
       type = "F4";
   }
-  else if(size == 65536)  // 64K
+  else if(size == 64*1024)  // 64K
   {
     if(isProbably3E(image, size))
       type = "3E";
@@ -444,6 +497,40 @@ bool Cartridge::isProbablySC(const uInt8* image, uInt32 size)
   return true;
 }
 
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+bool Cartridge::isProbably0840(const uInt8* image, uInt32 size)
+{
+  // 0840 cart bankswitching is triggered by accessing addresses 0x0800
+  // or 0x0840
+  uInt8 signature1[2][3] = {
+    { 0xAD, 0x00, 0x08 },  // LDA $0800
+    { 0xAD, 0x40, 0x08 }   // LDA $0840
+  };
+  for(uInt32 i = 0; i < 2; ++i)
+    if(searchForBytes(image, size, signature1[i], 3, 1))
+      return true;
+
+  uInt8 signature2[2][4] = {
+    { 0x0C, 0x00, 0x08, 0x4C },  // NOP $0800; JMP ...
+    { 0x0C, 0xFF, 0x0F, 0x4C }   // NOP $0FFF; JMP ...
+  };
+  for(uInt32 i = 0; i < 2; ++i)
+    if(searchForBytes(image, size, signature2[i], 4, 1))
+      return true;
+
+  return false;
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+bool Cartridge::isProbably3E(const uInt8* image, uInt32 size)
+{
+  // 3E cart bankswitching is triggered by storing the bank number
+  // in address 3E using 'STA $3E', commonly followed by an
+  // immediate mode LDA
+  uInt8 signature[] = { 0x85, 0x3E, 0xA9, 0x00 };  // STA $3E; LDA #$00
+  return searchForBytes(image, size, signature, 4, 1);
+}
+
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 bool Cartridge::isProbably3F(const uInt8* image, uInt32 size)
 {
@@ -456,13 +543,36 @@ bool Cartridge::isProbably3F(const uInt8* image, uInt32 size)
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-bool Cartridge::isProbably3E(const uInt8* image, uInt32 size)
+bool Cartridge::isProbably4A50(const uInt8* image, uInt32 size)
 {
-  // 3E cart bankswitching is triggered by storing the bank number
-  // in address 3E using 'STA $3E', commonly followed by an
-  // immediate mode LDA
-  uInt8 signature[] = { 0x85, 0x3E, 0xA9, 0x00 };  // STA $3E; LDA #$00
-  return searchForBytes(image, size, signature, 4, 1);
+  // 4A50 carts store address $4A50 at the NMI vector, which
+  // in this scheme is always in the last page of ROM at
+  // $1FFA - $1FFB (at least this is true in rev 1 of the format)
+  int idx = size - 6;  // $1FFA
+  return (image[idx] == 0x50 && image[idx+1] == 0x4A);
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+bool Cartridge::isProbablyCV(const uInt8* image, uInt32 size)
+{
+  // CV RAM access occurs at addresses $f3ff and $f400
+  // These signatures are attributed to the MESS project
+  uInt8 signature[2][3] = {
+    { 0x9D, 0xFF, 0xF3 },  // STA $F3FF
+    { 0x99, 0x00, 0xF4 }   // STA $F400
+  };
+  if(searchForBytes(image, size, signature[0], 3, 1))
+    return true;
+  else
+    return searchForBytes(image, size, signature[1], 3, 1);
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+bool Cartridge::isProbablyDPCplus(const uInt8* image, uInt32 size)
+{
+  // DPC+ ARM code has 2 occurrences of the string DPC+
+  uInt8 signature[] = { 0x44, 0x50, 0x43, 0x2B };  // DPC+
+  return searchForBytes(image, size, signature, 4, 2);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -500,14 +610,16 @@ bool Cartridge::isProbablyE7(const uInt8* image, uInt32 size)
   // search for only certain known signatures
   // Thanks to "stella@casperkitty.com" for this advice
   // These signatures are attributed to the MESS project
-  uInt8 signature[5][3] = {
+  uInt8 signature[7][3] = {
+   { 0xAD, 0xE2, 0xFF },  // LDA $FFE2
    { 0xAD, 0xE5, 0xFF },  // LDA $FFE5
    { 0xAD, 0xE5, 0x1F },  // LDA $1FE5
+   { 0xAD, 0xE7, 0x1F },  // LDA $1FE7
    { 0x0C, 0xE7, 0x1F },  // NOP $1FE7
    { 0x8D, 0xE7, 0xFF },  // STA $FFE7
    { 0x8D, 0xE7, 0x1F }   // STA $1FE7
   };
-  for(uInt32 i = 0; i < 5; ++i)
+  for(uInt32 i = 0; i < 7; ++i)
     if(searchForBytes(image, size, signature[i], 3, 1))
       return true;
 
@@ -530,86 +642,6 @@ bool Cartridge::isProbablyEF(const uInt8* image, uInt32 size)
     return searchForBytes(image, size, signature[1], 3, 1);
 }
 
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-bool Cartridge::isProbablyUA(const uInt8* image, uInt32 size)
-{
-  // UA cart bankswitching switches to bank 1 by accessing address 0x240
-  // using 'STA $240' or 'LDA $240'
-  uInt8 signature[3][3] = {
-    { 0x8D, 0x40, 0x02 },  // STA $240
-    { 0xAD, 0x40, 0x02 },  // LDA $240
-    { 0xBD, 0x1F, 0x02 }   // LDA $21F,X
-  };
-  for(uInt32 i = 0; i < 3; ++i)
-    if(searchForBytes(image, size, signature[i], 3, 1))
-      return true;
-
-  return false;
-}
-
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-bool Cartridge::isProbably4A50(const uInt8* image, uInt32 size)
-{
-  // 4A50 carts store address $4A50 at the NMI vector, which
-  // in this scheme is always in the last page of ROM at
-  // $1FFA - $1FFB (at least this is true in rev 1 of the format)
-  int idx = size - 6;  // $1FFA
-  return (image[idx] == 0x50 && image[idx+1] == 0x4A);
-}
-
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-bool Cartridge::isProbablySB(const uInt8* image, uInt32 size)
-{
-  // SB cart bankswitching switches banks by accessing address 0x0800
-  uInt8 signature[2][3] = {
-    { 0xBD, 0x00, 0x08 },  // LDA $0800,x
-    { 0xAD, 0x00, 0x08 }   // LDA $0800
-  };
-  if(searchForBytes(image, size, signature[0], 3, 1))
-    return true;
-  else
-    return searchForBytes(image, size, signature[1], 3, 1);
-}
-
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-bool Cartridge::isProbably0840(const uInt8* image, uInt32 size)
-{
-  // 0840 cart bankswitching is triggered by accessing addresses 0x0800
-  // or 0x0840
-  uInt8 signature1[2][3] = {
-    { 0xAD, 0x00, 0x08 },  // LDA $0800
-    { 0xAD, 0x40, 0x08 }   // LDA $0840
-  };
-  for(uInt32 i = 0; i < 2; ++i)
-    if(searchForBytes(image, size, signature1[i], 3, 1))
-      return true;
-
-  uInt8 signature2[2][4] = {
-    { 0x0C, 0x00, 0x08, 0x4C },  // NOP $0800; JMP ...
-    { 0x0C, 0xFF, 0x0F, 0x4C }   // NOP $0FFF; JMP ...
-  };
-  for(uInt32 i = 0; i < 2; ++i)
-    if(searchForBytes(image, size, signature2[i], 4, 1))
-      return true;
-
-  return false;
-}
-
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-bool Cartridge::isProbablyCV(const uInt8* image, uInt32 size)
-{
-  // CV RAM access occurs at addresses $f3ff and $f400
-  // These signatures are attributed to the MESS project
-  uInt8 signature[2][3] = {
-    { 0x9D, 0xFF, 0xF3 },  // STA $F3FF
-    { 0x99, 0x00, 0xF4 }   // STA $F400
-  };
-  if(searchForBytes(image, size, signature[0], 3, 1))
-    return true;
-  else
-    return searchForBytes(image, size, signature[1], 3, 1);
-}
-
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 bool Cartridge::isProbablyFE(const uInt8* image, uInt32 size)
 {
@@ -629,6 +661,37 @@ bool Cartridge::isProbablyFE(const uInt8* image, uInt32 size)
   return false;
 }
 
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+bool Cartridge::isProbablySB(const uInt8* image, uInt32 size)
+{
+  // SB cart bankswitching switches banks by accessing address 0x0800
+  uInt8 signature[2][3] = {
+    { 0xBD, 0x00, 0x08 },  // LDA $0800,x
+    { 0xAD, 0x00, 0x08 }   // LDA $0800
+  };
+  if(searchForBytes(image, size, signature[0], 3, 1))
+    return true;
+  else
+    return searchForBytes(image, size, signature[1], 3, 1);
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+bool Cartridge::isProbablyUA(const uInt8* image, uInt32 size)
+{
+  // UA cart bankswitching switches to bank 1 by accessing address 0x240
+  // using 'STA $240' or 'LDA $240'
+  uInt8 signature[3][3] = {
+    { 0x8D, 0x40, 0x02 },  // STA $240
+    { 0xAD, 0x40, 0x02 },  // LDA $240
+    { 0xBD, 0x1F, 0x02 }   // LDA $21F,X
+  };
+  for(uInt32 i = 0; i < 3; ++i)
+    if(searchForBytes(image, size, signature[i], 3, 1))
+      return true;
+
+  return false;
+}
+
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 bool Cartridge::isProbablyX07(const uInt8* image, uInt32 size)
 {
@@ -646,7 +709,8 @@ bool Cartridge::isProbablyX07(const uInt8* image, uInt32 size)
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-Cartridge::Cartridge(const Cartridge&)
+Cartridge::Cartridge(const Cartridge& cart)
+  : mySettings(cart.mySettings)
 {
   assert(false);
 }
