@@ -8,12 +8,13 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2009 by Bradford W. Mott and the Stella team
+// Copyright (c) 1995-2010 by Bradford W. Mott, Stephen Anthony
+// and the Stella Team
 //
-// See the file "license" for information on usage and redistribution of
+// See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: Cart2K.cxx 1862 2009-08-27 22:59:14Z stephena $
+// $Id: Cart2K.cxx 2138 2010-10-03 00:23:13Z stephena $
 //============================================================================
 
 #include <cassert>
@@ -23,7 +24,8 @@
 #include "Cart2K.hxx"
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-Cartridge2K::Cartridge2K(const uInt8* image, uInt32 size)
+Cartridge2K::Cartridge2K(const uInt8* image, uInt32 size, const Settings& settings)
+  : Cartridge(settings)
 {
   // Size can be a maximum of 2K
   if(size > 2048) size = 2048;
@@ -45,6 +47,7 @@ Cartridge2K::Cartridge2K(const uInt8* image, uInt32 size)
 
   // Copy the ROM image into my buffer
   memcpy(myImage, image, size);
+  createCodeAccessBase(mySize);
 
   // Set mask for accessing the image buffer
   // This is guaranteed to work, as mySize is a power of two
@@ -60,6 +63,7 @@ Cartridge2K::~Cartridge2K()
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void Cartridge2K::reset()
 {
+  myBankChanged = true;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -73,12 +77,12 @@ void Cartridge2K::install(System& system)
   assert((0x1000 & mask) == 0);
 
   // Map ROM image into the system
-  System::PageAccess access;
+  System::PageAccess access(0, 0, 0, this, System::PA_READ);
+
   for(uInt32 address = 0x1000; address < 0x2000; address += (1 << shift))
   {
-    access.device = this;
     access.directPeekBase = &myImage[address & myMask];
-    access.directPokeBase = 0;
+    access.codeAccessBase = &myCodeAccessBase[address & myMask];
     mySystem->setPageAccess(address >> shift, access);
   }
 }
@@ -90,26 +94,28 @@ uInt8 Cartridge2K::peek(uInt16 address)
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void Cartridge2K::poke(uInt16, uInt8)
+bool Cartridge2K::poke(uInt16, uInt8)
 {
   // This is ROM so poking has no effect :-)
+  return false;
 } 
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void Cartridge2K::bank(uInt16 bank)
+bool Cartridge2K::bank(uInt16 bank)
 {
   // Doesn't support bankswitching
+  return false;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-int Cartridge2K::bank()
+uInt16 Cartridge2K::bank() const
 {
   // Doesn't support bankswitching
   return 0;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-int Cartridge2K::bankCount()
+uInt16 Cartridge2K::bankCount() const
 {
   return 1;
 }
@@ -118,24 +124,22 @@ int Cartridge2K::bankCount()
 bool Cartridge2K::patch(uInt16 address, uInt8 value)
 {
   myImage[address & myMask] = value;
-  return true;
+  return myBankChanged = true;
 } 
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-uInt8* Cartridge2K::getImage(int& size)
+const uInt8* Cartridge2K::getImage(int& size) const
 {
   size = mySize;
-  return &myImage[0];
+  return myImage;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 bool Cartridge2K::save(Serializer& out) const
 {
-  const string& cart = name();
-
   try
   {
-    out.putString(cart);
+    out.putString(name());
   }
   catch(const char* msg)
   {
@@ -149,11 +153,9 @@ bool Cartridge2K::save(Serializer& out) const
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 bool Cartridge2K::load(Serializer& in)
 {
-  const string& cart = name();
-
   try
   {
-    if(in.getString() != cart)
+    if(in.getString() != name())
       return false;
   }
   catch(const char* msg)
