@@ -8,39 +8,45 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2009 by Bradford W. Mott and the Stella team
+// Copyright (c) 1995-2010 by Bradford W. Mott, Stephen Anthony
+// and the Stella Team
 //
-// See the file "license" for information on usage and redistribution of
+// See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: CartAR.cxx 1862 2009-08-27 22:59:14Z stephena $
+// $Id: CartAR.cxx 2184 2010-11-11 22:24:51Z stephena $
 //============================================================================
 
 #include <cassert>
 #include <cstring>
 
 #include "M6502.hxx"
-#include "Random.hxx"
 #include "System.hxx"
 #include "CartAR.hxx"
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 CartridgeAR::CartridgeAR(const uInt8* image, uInt32 size,
                          const Settings& settings)
-  : my6502(0),
-    mySettings(settings)
+  : Cartridge(settings),
+    my6502(0),
+    mySize(BSPF_max(size, 8448u))
 {
-  // Minimum size supported internally is 8448 bytes
-  uInt32 minsize = BSPF_max(size, 8448u);
-
   // Create a load image buffer and copy the given image
-  myLoadImages = new uInt8[minsize];
-  myNumberOfLoadImages = minsize / 8448;
+  myLoadImages = new uInt8[mySize];
+  myNumberOfLoadImages = mySize / 8448;
   memcpy(myLoadImages, image, size);
 
   // Add header if image doesn't include it
   if(size < 8448)
     memcpy(myLoadImages+8192, ourDefaultHeader, 256);
+
+  // We use System::PageAccess.codeAccessBase, but don't allow its use
+  // through a pointer, since the AR scheme doesn't support bankswitching
+  // in the normal sense
+  //
+  // Instead, access will be through the getAccessFlags and setAccessFlags
+  // methods below
+  createCodeAccessBase(mySize);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -52,10 +58,12 @@ CartridgeAR::~CartridgeAR()
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void CartridgeAR::reset()
 {
-  // Initialize RAM with random values
-  class Random random;
-  for(uInt32 i = 0; i < 6 * 1024; ++i)
-    myImage[i] = random.next();
+  // Initialize RAM
+  if(mySettings.getBool("ramrandom"))
+    for(uInt32 i = 0; i < 6 * 1024; ++i)
+      myImage[i] = mySystem->randGenerator().next();
+  else
+    memset(myImage, 0, 6 * 1024);
 
   // Initialize SC BIOS ROM
   initializeROM();
@@ -94,14 +102,11 @@ void CartridgeAR::install(System& system)
   // Make sure the system we're being installed in has a page size that'll work
   assert((0x1000 & mask) == 0);
 
-  System::PageAccess access;
+  // Map all of the accesses to call peek and poke (we don't yet indicate RAM areas)
+  System::PageAccess access(0, 0, 0, this, System::PA_READ);
+
   for(uInt32 i = 0x1000; i < 0x2000; i += (1 << shift))
-  {
-    access.directPeekBase = 0;
-    access.directPokeBase = 0;
-    access.device = this;
     mySystem->setPageAccess(i >> shift, access);
-  }
 
   bankConfiguration(0);
 }
@@ -109,8 +114,13 @@ void CartridgeAR::install(System& system)
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 uInt8 CartridgeAR::peek(uInt16 addr)
 {
+  // In debugger/bank-locked mode, we ignore all hotspots and in general
+  // anything that can change the internal state of the cart
+  if(bankLocked())
+    return myImage[(addr & 0x07FF) + myImageOffset[(addr & 0x0800) ? 1 : 0]];
+
   // Is the "dummy" SC BIOS hotspot for reading a load being accessed?
-  if(((addr & 0x1FFF) == 0x1850) && (myImageOffset[1] == (3 * 2048)))
+  if(((addr & 0x1FFF) == 0x1850) && (myImageOffset[1] == (3 << 11)))
   {
     // Get load that's being accessed (BIOS places load number at 0x80)
     uInt8 load = mySystem->peek(0x0080);
@@ -148,9 +158,15 @@ uInt8 CartridgeAR::peek(uInt16 addr)
       (my6502->distinctAccesses() == (myNumberOfDistinctAccesses + 5)))
   {
     if((addr & 0x0800) == 0)
+    {
       myImage[(addr & 0x07FF) + myImageOffset[0]] = myDataHoldRegister;
-    else if(myImageOffset[1] != 3 * 2048)    // Can't poke to ROM :-)
+      mySystem->setDirtyPage(addr);
+    }
+    else if(myImageOffset[1] != (3 << 11))    // Can't poke to ROM :-)
+    {
       myImage[(addr & 0x07FF) + myImageOffset[1]] = myDataHoldRegister;
+      mySystem->setDirtyPage(addr);
+    }
     myWritePending = false;
   }
 
@@ -158,8 +174,10 @@ uInt8 CartridgeAR::peek(uInt16 addr)
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void CartridgeAR::poke(uInt16 addr, uInt8)
+bool CartridgeAR::poke(uInt16 addr, uInt8)
 {
+  bool modified = false;
+
   // Cancel any pending write if more than 5 distinct accesses have occurred
   // TODO: Modify to handle when the distinct counter wraps around...
   if(myWritePending && 
@@ -187,15 +205,37 @@ void CartridgeAR::poke(uInt16 addr, uInt8)
       (my6502->distinctAccesses() == (myNumberOfDistinctAccesses + 5)))
   {
     if((addr & 0x0800) == 0)
+    {
       myImage[(addr & 0x07FF) + myImageOffset[0]] = myDataHoldRegister;
-    else if(myImageOffset[1] != 3 * 2048)    // Can't poke to ROM :-)
+      modified = true;
+    }
+    else if(myImageOffset[1] != (3 << 11))    // Can't poke to ROM :-)
+    {
       myImage[(addr & 0x07FF) + myImageOffset[1]] = myDataHoldRegister;
+      modified = true;
+    }
     myWritePending = false;
   }
+
+  return modified;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void CartridgeAR::bankConfiguration(uInt8 configuration)
+uInt8 CartridgeAR::getAccessFlags(uInt16 address)
+{
+  return myCodeAccessBase[(address & 0x07FF) +
+           myImageOffset[(address & 0x0800) ? 1 : 0]];
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void CartridgeAR::setAccessFlags(uInt16 address, uInt8 flags)
+{
+  myCodeAccessBase[(address & 0x07FF) +
+    myImageOffset[(address & 0x0800) ? 1 : 0]] |= flags;
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+bool CartridgeAR::bankConfiguration(uInt8 configuration)
 {
   // D7-D5 of this byte: Write Pulse Delay (n/a for emulator)
   //
@@ -215,7 +255,7 @@ void CartridgeAR::bankConfiguration(uInt8 configuration)
   //  p = ROM Power (0 = enabled, 1 = off.)  Only power the ROM if you're
   //    wanting to access the ROM for multiloads.  Otherwise set to 1.
 
-  myCurrentBank = configuration & 0x1f; // remember for the bank() method
+  myCurrentBank = configuration & 0x1F; // remember for the bank() method
 
   // Handle ROM power configuration
   myPower = !(configuration & 0x01);
@@ -231,60 +271,61 @@ void CartridgeAR::bankConfiguration(uInt8 configuration)
   {
     case 0:
     {
-      myImageOffset[0] = 2 * 2048;
-      myImageOffset[1] = 3 * 2048;
+      myImageOffset[0] = 2 << 11;
+      myImageOffset[1] = 3 << 11;
       break;
     }
 
     case 1:
     {
-      myImageOffset[0] = 0 * 2048;
-      myImageOffset[1] = 3 * 2048;
+      myImageOffset[0] = 0      ;
+      myImageOffset[1] = 3 << 11;
       break;
     }
 
     case 2:
     {
-      myImageOffset[0] = 2 * 2048;
-      myImageOffset[1] = 0 * 2048;
+      myImageOffset[0] = 2 << 11;
+      myImageOffset[1] = 0      ;
       break;
     }
 
     case 3:
     {
-      myImageOffset[0] = 0 * 2048;
-      myImageOffset[1] = 2 * 2048;
+      myImageOffset[0] = 0      ;
+      myImageOffset[1] = 2 << 11;
       break;
     }
 
     case 4:
     {
-      myImageOffset[0] = 2 * 2048;
-      myImageOffset[1] = 3 * 2048;
+      myImageOffset[0] = 2 << 11;
+      myImageOffset[1] = 3 << 11;
       break;
     }
 
     case 5:
     {
-      myImageOffset[0] = 1 * 2048;
-      myImageOffset[1] = 3 * 2048;
+      myImageOffset[0] = 1 << 11;
+      myImageOffset[1] = 3 << 11;
       break;
     }
 
     case 6:
     {
-      myImageOffset[0] = 2 * 2048;
-      myImageOffset[1] = 1 * 2048;
+      myImageOffset[0] = 2 << 11;
+      myImageOffset[1] = 1 << 11;
       break;
     }
 
     case 7:
     {
-      myImageOffset[0] = 1 * 2048;
-      myImageOffset[1] = 2 * 2048;
+      myImageOffset[0] = 1 << 11;
+      myImageOffset[1] = 2 << 11;
       break;
     }
   }
+  return myBankChanged = true;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -296,28 +337,25 @@ void CartridgeAR::initializeROM()
   // almost definitely change
 
   // The scrom.asm code checks a value at offset 109 as follows:
-  //   0xff -> do a complete jump over the SC BIOS progress bars code
-  //   0x0  -> show SC BIOS progress bars as normal
-  ourDummyROMCode[109] = mySettings.getBool("fastscbios") ? 0xff : 0x0;
+  //   0xFF -> do a complete jump over the SC BIOS progress bars code
+  //   0x00 -> show SC BIOS progress bars as normal
+  ourDummyROMCode[109] = mySettings.getBool("fastscbios") ? 0xFF : 0x00;
 
   // The accumulator should contain a random value after exiting the
   // SC BIOS code - a value placed in offset 281 will be stored in A
-  class Random random;
-  ourDummyROMCode[281] = random.next();
+  ourDummyROMCode[281] = mySystem->randGenerator().next();
 
   // Initialize ROM with illegal 6502 opcode that causes a real 6502 to jam
-  for(uInt32 i = 0; i < 2048; ++i)
-    myImage[3 * 2048 + i] = 0x02;
+  memset(myImage + (3<<11), 0x02, 2048);
 
   // Copy the "dummy" Supercharger BIOS code into the ROM area
-  for(uInt32 j = 0; j < sizeof(ourDummyROMCode); ++j)
-    myImage[3 * 2048 + j] = ourDummyROMCode[j];
+  memcpy(myImage + (3<<11), ourDummyROMCode, sizeof(ourDummyROMCode));
 
   // Finally set 6502 vectors to point to initial load code at 0xF80A of BIOS
-  myImage[3 * 2048 + 2044] = 0x0A;
-  myImage[3 * 2048 + 2045] = 0xF8;
-  myImage[3 * 2048 + 2046] = 0x0A;
-  myImage[3 * 2048 + 2047] = 0xF8;
+  myImage[(3<<11) + 2044] = 0x0A;
+  myImage[(3<<11) + 2045] = 0xF8;
+  myImage[(3<<11) + 2046] = 0x0A;
+  myImage[(3<<11) + 2047] = 0xF8;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -379,6 +417,7 @@ void CartridgeAR::loadIntoRAM(uInt8 load)
       mySystem->poke(0xff, myHeader[1]);
       mySystem->poke(0x80, myHeader[2]);
 
+      myBankChanged = true;
       return;
     }
   }
@@ -389,21 +428,22 @@ void CartridgeAR::loadIntoRAM(uInt8 load)
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void CartridgeAR::bank(uInt16 bank)
+bool CartridgeAR::bank(uInt16 bank)
 {
-  if(myBankLocked) return;
-
-  bankConfiguration(bank);
+  if(!bankLocked())
+    return bankConfiguration(bank);
+  else
+    return false;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-int CartridgeAR::bank()
+uInt16 CartridgeAR::bank() const
 {
   return myCurrentBank;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-int CartridgeAR::bankCount()
+uInt16 CartridgeAR::bankCount() const
 {
   return 32;
 }
@@ -416,22 +456,20 @@ bool CartridgeAR::patch(uInt16 address, uInt8 value)
 } 
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-uInt8* CartridgeAR::getImage(int& size)
+const uInt8* CartridgeAR::getImage(int& size) const
 {
-  size = myNumberOfLoadImages * 8448;
-  return &myLoadImages[0];
+  size = mySize;
+  return myLoadImages;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 bool CartridgeAR::save(Serializer& out) const
 {
-  const string& cart = name();
-
   try
   {
     uInt32 i;
 
-    out.putString(cart);
+    out.putString(name());
 
     // Indicates the offest within the image for the corresponding bank
     out.putInt(2);
@@ -487,11 +525,9 @@ bool CartridgeAR::save(Serializer& out) const
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 bool CartridgeAR::load(Serializer& in)
 {
-  const string& cart = name();
-
   try
   {
-    if(in.getString() != cart)
+    if(in.getString() != name())
       return false;
 
     uInt32 i, limit;
