@@ -8,33 +8,37 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2009 by Bradford W. Mott and the Stella team
+// Copyright (c) 1995-2010 by Bradford W. Mott, Stephen Anthony
+// and the Stella Team
 //
-// See the file "license" for information on usage and redistribution of
+// See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: CartDPC.cxx 1862 2009-08-27 22:59:14Z stephena $
+// $Id: CartDPC.cxx 2145 2010-10-10 20:24:22Z stephena $
 //============================================================================
 
 #include <cassert>
 #include <cstring>
-#include <iostream>
 
 #include "System.hxx"
 #include "CartDPC.hxx"
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-CartridgeDPC::CartridgeDPC(const uInt8* image, uInt32 size)
+CartridgeDPC::CartridgeDPC(const uInt8* image, uInt32 size,
+                           const Settings& settings)
+  : Cartridge(settings),
+    mySystemCycles(0),
+    myFractionalClocks(0.0)
 {
-  // Make a copy of the entire image as-is, for use by getImage()
-  // (this wastes 12K of RAM, should be controlled by a #ifdef)
-  memcpy(myImageCopy, image, size);
+  // Make a copy of the entire image
+  memcpy(myImage, image, BSPF_min(size, 8192u + 2048u + 255u));
+  createCodeAccessBase(8192);
 
-  // Copy the program ROM image into my buffer
-  memcpy(myProgramImage, image, 8192);
+  // Pointer to the program ROM (8K @ 0 byte offset)
+  myProgramImage = myImage;
 
-  // Copy the display ROM image into my buffer
-  memcpy(myDisplayImage, image + 8192, 2048);
+  // Pointer to the display ROM (2K @ 8K offset)
+  myDisplayImage = myProgramImage + 8192;
 
   // Initialize the DPC data fetcher registers
   for(uInt16 i = 0; i < 8; ++i)
@@ -46,9 +50,8 @@ CartridgeDPC::CartridgeDPC(const uInt8* image, uInt32 size)
   // Initialize the DPC's random number generator register (must be non-zero)
   myRandomNumber = 1;
 
-  // Initialize the system cycles counter & fractional clock values
-  mySystemCycles = 0;
-  myFractionalClocks = 0.0;
+  // Remember startup bank
+  myStartBank = 1;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -63,8 +66,8 @@ void CartridgeDPC::reset()
   mySystemCycles = mySystem->cycles();
   myFractionalClocks = 0.0;
 
-  // Upon reset we switch to bank 1
-  bank(1);
+  // Upon reset we switch to the startup bank
+  bank(myStartBank);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -87,27 +90,15 @@ void CartridgeDPC::install(System& system)
   // Make sure the system we're being installed in has a page size that'll work
   assert(((0x1080 & mask) == 0) && ((0x1100 & mask) == 0));
 
-  // Set the page accessing methods for the hot spots
-  System::PageAccess access;
-  for(uInt32 i = (0x1FF8 & ~mask); i < 0x2000; i += (1 << shift))
-  {
-    access.directPeekBase = 0;
-    access.directPokeBase = 0;
-    access.device = this;
-    mySystem->setPageAccess(i >> shift, access);
-  }
+  System::PageAccess access(0, 0, 0, this, System::PA_READ);
 
   // Set the page accessing method for the DPC reading & writing pages
+  access.type = System::PA_READWRITE;
   for(uInt32 j = 0x1000; j < 0x1080; j += (1 << shift))
-  {
-    access.directPeekBase = 0;
-    access.directPokeBase = 0;
-    access.device = this;
     mySystem->setPageAccess(j >> shift, access);
-  }
 
-  // Install pages for bank 1
-  bank(1);
+  // Install pages for the startup bank
+  bank(myStartBank);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -187,6 +178,11 @@ uInt8 CartridgeDPC::peek(uInt16 address)
 {
   address &= 0x0FFF;
 
+  // In debugger/bank-locked mode, we ignore all hotspots and in general
+  // anything that can change the internal state of the cart
+  if(bankLocked())
+    return myProgramImage[(myCurrentBank << 12) + address];
+
   // Clock the random number generator.  This should be done for every
   // cartridge access, however, we're only doing it for the DPC and 
   // hot-spot accesses to save time.
@@ -301,12 +297,12 @@ uInt8 CartridgeDPC::peek(uInt16 address)
       default:
         break;
     }
-    return myProgramImage[myCurrentBank * 4096 + address];
+    return myProgramImage[(myCurrentBank << 12) + address];
   }
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void CartridgeDPC::poke(uInt16 address, uInt8 value)
+bool CartridgeDPC::poke(uInt16 address, uInt8 value)
 {
   address &= 0x0FFF;
 
@@ -343,14 +339,14 @@ void CartridgeDPC::poke(uInt16 address, uInt8 value)
       {
         if((index >= 5) && myMusicMode[index - 5])
         {
-          // Data fecther is in music mode so its low counter value
+          // Data fetcher is in music mode so its low counter value
           // should be loaded from the top register not the poked value
           myCounters[index] = (myCounters[index] & 0x0700) |
               (uInt16)myTops[index];
         }
         else
         {
-          // Data fecther is either not a music mode data fecther or it
+          // Data fetcher is either not a music mode data fetcher or it
           // isn't in music mode so it's low counter value should be loaded
           // with the poked value
           myCounters[index] = (myCounters[index] & 0x0700) | (uInt16)value;
@@ -408,79 +404,82 @@ void CartridgeDPC::poke(uInt16 address, uInt8 value)
         break;
     }
   }
+  return false;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void CartridgeDPC::bank(uInt16 bank)
+bool CartridgeDPC::bank(uInt16 bank)
 { 
-  if(myBankLocked) return;
+  if(bankLocked()) return false;
 
   // Remember what bank we're in
   myCurrentBank = bank;
-  uInt16 offset = myCurrentBank * 4096;
+  uInt16 offset = myCurrentBank << 12;
   uInt16 shift = mySystem->pageShift();
   uInt16 mask = mySystem->pageMask();
 
-  // Setup the page access methods for the current bank
-  System::PageAccess access;
-  access.device = this;
-  access.directPokeBase = 0;
+  System::PageAccess access(0, 0, 0, this, System::PA_READ);
 
-  // Map Program ROM image into the system
+  // Set the page accessing methods for the hot spots
+  for(uInt32 i = (0x1FF8 & ~mask); i < 0x2000; i += (1 << shift))
+  {
+    access.codeAccessBase = &myCodeAccessBase[offset + (i & 0x0FFF)];
+    mySystem->setPageAccess(i >> shift, access);
+  }
+
+  // Setup the page access methods for the current bank
   for(uInt32 address = 0x1080; address < (0x1FF8U & ~mask);
       address += (1 << shift))
   {
     access.directPeekBase = &myProgramImage[offset + (address & 0x0FFF)];
+    access.codeAccessBase = &myCodeAccessBase[offset + (address & 0x0FFF)];
     mySystem->setPageAccess(address >> shift, access);
   }
+  return myBankChanged = true;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-int CartridgeDPC::bank()
+uInt16 CartridgeDPC::bank() const
 {
   return myCurrentBank;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-int CartridgeDPC::bankCount()
+uInt16 CartridgeDPC::bankCount() const
 {
-  // TODO - add support for debugger (support the display ROM somehow)
   return 2;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 bool CartridgeDPC::patch(uInt16 address, uInt8 value)
 {
-  // TODO - check if this actually works
-  myProgramImage[(myCurrentBank << 12) + (address & 0x0FFF)] = value;
-  return true;
+  address &= 0x0FFF;
+
+  // For now, we ignore attempts to patch the DPC address space
+  if(address >= 0x0080)
+  {
+    myProgramImage[(myCurrentBank << 12) + (address & 0x0FFF)] = value;
+    return myBankChanged = true;
+  }
+  else
+    return false;
 } 
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-uInt8* CartridgeDPC::getImage(int& size)
+const uInt8* CartridgeDPC::getImage(int& size) const
 {
   size = 8192 + 2048 + 255;
-
-  int i;
-  for(i = 0; i < 8192; i++)
-    myImageCopy[i] = myProgramImage[i];
-
-  for(i = 0; i < 2048; i++)
-    myImageCopy[i + 8192] = myDisplayImage[i];
-
-  return &myImageCopy[0];
+  return myImage;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 bool CartridgeDPC::save(Serializer& out) const
 {
-  const string& cart = name();
-
   try
   {
     uInt32 i;
 
-    out.putString(cart);
+    out.putString(name());
 
     // Indicates which bank is currently active
     out.putInt(myCurrentBank);
@@ -528,11 +527,9 @@ bool CartridgeDPC::save(Serializer& out) const
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 bool CartridgeDPC::load(Serializer& in)
 {
-  const string& cart = name();
-
   try
   {
-    if(in.getString() != cart)
+    if(in.getString() != name())
       return false;
 
     uInt32 i, limit;
