@@ -8,12 +8,13 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2009 by Bradford W. Mott and the Stella team
+// Copyright (c) 1995-2010 by Bradford W. Mott, Stephen Anthony
+// and the Stella Team
 //
-// See the file "license" for information on usage and redistribution of
+// See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: CartE0.cxx 1862 2009-08-27 22:59:14Z stephena $
+// $Id: CartE0.cxx 2145 2010-10-10 20:24:22Z stephena $
 //============================================================================
 
 #include <cassert>
@@ -23,10 +24,12 @@
 #include "CartE0.hxx"
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-CartridgeE0::CartridgeE0(const uInt8* image)
+CartridgeE0::CartridgeE0(const uInt8* image, const Settings& settings)
+  : Cartridge(settings)
 {
   // Copy the ROM image into my buffer
   memcpy(myImage, image, 8192);
+  createCodeAccessBase(8192);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -41,6 +44,8 @@ void CartridgeE0::reset()
   segmentZero(4);
   segmentOne(5);
   segmentTwo(6);
+
+  myBankChanged = true;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -54,25 +59,23 @@ void CartridgeE0::install(System& system)
   assert(((0x1000 & mask) == 0) && ((0x1400 & mask) == 0) &&
       ((0x1800 & mask) == 0) && ((0x1C00 & mask) == 0));
 
+  System::PageAccess access(0, 0, 0, this, System::PA_READ);
+
   // Set the page acessing methods for the first part of the last segment
-  System::PageAccess access;
-  access.directPokeBase = 0;
-  access.device = this;
   for(uInt32 i = 0x1C00; i < (0x1FE0U & ~mask); i += (1 << shift))
   {
     access.directPeekBase = &myImage[7168 + (i & 0x03FF)];
+    access.codeAccessBase = &myCodeAccessBase[7168 + (i & 0x03FF)];
     mySystem->setPageAccess(i >> shift, access);
   }
   myCurrentSlice[3] = 7;
 
   // Set the page accessing methods for the hot spots in the last segment
   access.directPeekBase = 0;
-  access.directPokeBase = 0;
-  access.device = this;
+  access.codeAccessBase = &myCodeAccessBase[8128];
+  access.type = System::PA_READ;
   for(uInt32 j = (0x1FE0 & ~mask); j < 0x2000; j += (1 << shift))
-  {
     mySystem->setPageAccess(j >> shift, access);
-  }
 
   // Install some default slices for the other segments
   segmentZero(4);
@@ -103,7 +106,7 @@ uInt8 CartridgeE0::peek(uInt16 address)
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void CartridgeE0::poke(uInt16 address, uInt8)
+bool CartridgeE0::poke(uInt16 address, uInt8)
 {
   address &= 0x0FFF;
 
@@ -120,12 +123,13 @@ void CartridgeE0::poke(uInt16 address, uInt8)
   {
     segmentTwo(address & 0x0007);
   }
+  return false;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void CartridgeE0::segmentZero(uInt16 slice)
 { 
-  if(myBankLocked) return;
+  if(bankLocked()) return;
 
   // Remember the new slice
   myCurrentSlice[0] = slice;
@@ -133,21 +137,21 @@ void CartridgeE0::segmentZero(uInt16 slice)
   uInt16 shift = mySystem->pageShift();
 
   // Setup the page access methods for the current bank
-  System::PageAccess access;
-  access.device = this;
-  access.directPokeBase = 0;
+  System::PageAccess access(0, 0, 0, this, System::PA_READ);
 
   for(uInt32 address = 0x1000; address < 0x1400; address += (1 << shift))
   {
     access.directPeekBase = &myImage[offset + (address & 0x03FF)];
+    access.codeAccessBase = &myCodeAccessBase[offset + (address & 0x03FF)];
     mySystem->setPageAccess(address >> shift, access);
   }
+  myBankChanged = true;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void CartridgeE0::segmentOne(uInt16 slice)
 { 
-  if(myBankLocked) return;
+  if(bankLocked()) return;
 
   // Remember the new slice
   myCurrentSlice[1] = slice;
@@ -155,21 +159,21 @@ void CartridgeE0::segmentOne(uInt16 slice)
   uInt16 shift = mySystem->pageShift();
 
   // Setup the page access methods for the current bank
-  System::PageAccess access;
-  access.device = this;
-  access.directPokeBase = 0;
+  System::PageAccess access(0, 0, 0, this, System::PA_READ);
 
   for(uInt32 address = 0x1400; address < 0x1800; address += (1 << shift))
   {
     access.directPeekBase = &myImage[offset + (address & 0x03FF)];
+    access.codeAccessBase = &myCodeAccessBase[offset + (address & 0x03FF)];
     mySystem->setPageAccess(address >> shift, access);
   }
+  myBankChanged = true;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void CartridgeE0::segmentTwo(uInt16 slice)
 { 
-  if(myBankLocked) return;
+  if(bankLocked()) return;
 
   // Remember the new slice
   myCurrentSlice[2] = slice;
@@ -177,37 +181,36 @@ void CartridgeE0::segmentTwo(uInt16 slice)
   uInt16 shift = mySystem->pageShift();
 
   // Setup the page access methods for the current bank
-  System::PageAccess access;
-  access.device = this;
-  access.directPokeBase = 0;
+  System::PageAccess access(0, 0, 0, this, System::PA_READ);
 
   for(uInt32 address = 0x1800; address < 0x1C00; address += (1 << shift))
   {
     access.directPeekBase = &myImage[offset + (address & 0x03FF)];
+    access.codeAccessBase = &myCodeAccessBase[offset + (address & 0x03FF)];
     mySystem->setPageAccess(address >> shift, access);
   }
+  myBankChanged = true;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void CartridgeE0::bank(uInt16)
+bool CartridgeE0::bank(uInt16)
 {
   // Doesn't support bankswitching in the normal sense
-  // TODO - add support for debugger
+  return false;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-int CartridgeE0::bank()
+uInt16 CartridgeE0::bank() const
 {
   // Doesn't support bankswitching in the normal sense
-  // TODO - add support for debugger
   return 0;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-int CartridgeE0::bankCount()
+uInt16 CartridgeE0::bankCount() const
 {
   // Doesn't support bankswitching in the normal sense
-  // TODO - add support for debugger
+  // There is one 'virtual' bank that can change in many different ways
   return 1;
 }
 
@@ -220,20 +223,18 @@ bool CartridgeE0::patch(uInt16 address, uInt8 value)
 } 
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-uInt8* CartridgeE0::getImage(int& size)
+const uInt8* CartridgeE0::getImage(int& size) const
 {
   size = 8192;
-  return &myImage[0];
+  return myImage;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 bool CartridgeE0::save(Serializer& out) const
 {
-  const string& cart = name();
-
   try
   {
-    out.putString(cart);
+    out.putString(name());
 
     out.putInt(4);
     for(uInt32 i = 0; i < 4; ++i)
@@ -251,11 +252,9 @@ bool CartridgeE0::save(Serializer& out) const
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 bool CartridgeE0::load(Serializer& in)
 {
-  const string& cart = name();
-
   try
   {
-    if(in.getString() != cart)
+    if(in.getString() != name())
       return false;
 
     uInt32 limit = (uInt32) in.getInt();
