@@ -8,29 +8,30 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2009 by Bradford W. Mott and the Stella team
+// Copyright (c) 1995-2010 by Bradford W. Mott, Stephen Anthony
+// and the Stella Team
 //
-// See the file "license" for information on usage and redistribution of
+// See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: M6532.cxx 1862 2009-08-27 22:59:14Z stephena $
+// $Id: M6532.cxx 2138 2010-10-03 00:23:13Z stephena $
 //============================================================================
 
 #include <cassert>
 #include <iostream>
 
 #include "Console.hxx"
-#include "Random.hxx"
+#include "Settings.hxx"
 #include "Switches.hxx"
 #include "System.hxx"
 
 #include "M6532.hxx"
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-M6532::M6532(const Console& console)
-  : myConsole(console)
+M6532::M6532(const Console& console, const Settings& settings)
+  : myConsole(console),
+    mySettings(settings)
 {
-  reset();
 }
  
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -41,22 +42,23 @@ M6532::~M6532()
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void M6532::reset()
 {
-  class Random random;
-
-  // Randomize the 128 bytes of memory
-  for(uInt32 t = 0; t < 128; ++t)
-    myRAM[t] = random.next();
+  // Initialize the 128 bytes of memory
+  if(mySettings.getBool("ramrandom"))
+    for(uInt32 t = 0; t < 128; ++t)
+      myRAM[t] = mySystem->randGenerator().next();
+  else
+    memset(myRAM, 0, 128);
 
   // The timer absolutely cannot be initialized to zero; some games will
   // loop or hang (notably Solaris and H.E.R.O.)
-  myTimer = (0xff - (random.next() % 0xfe)) << 10;
+  myTimer = (0xff - (mySystem->randGenerator().next() % 0xfe)) << 10;
   myIntervalShift = 10;
   myCyclesWhenTimerSet = 0;
   myInterruptEnabled = false;
   myInterruptTriggered = false;
 
   // Zero the I/O registers
-  myDDRA = myDDRB = myOutA = 0x00;
+  myDDRA = myDDRB = myOutA = myOutB = 0x00;
 
   // Zero the timer registers
   myOutTimer[0] = myOutTimer[1] = myOutTimer[2] = myOutTimer[3] = 0x00;
@@ -93,19 +95,12 @@ void M6532::install(System& system, Device& device)
   assert((0x1080 & mask) == 0);
   
   // All accesses are to the given device
-  System::PageAccess access;
-  access.device = &device;
+  System::PageAccess access(0, 0, 0, &device, System::PA_READWRITE);
 
   // We're installing in a 2600 system
   for(int address = 0; address < 8192; address += (1 << shift))
-  {
     if((address & 0x1080) == 0x0080)
-    {
-      access.directPeekBase = 0; 
-      access.directPokeBase = 0;
       mySystem->setPageAccess(address >> shift, access);
-    }
-  }
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -122,7 +117,7 @@ uInt8 M6532::peek(uInt16 addr)
 
   switch(addr & 0x07)
   {
-    case 0x00:    // Port A I/O Register (Joystick)
+    case 0x00:    // SWCHA - Port A I/O Register (Joystick)
     {
       uInt8 value = 0x00;
 
@@ -145,17 +140,17 @@ uInt8 M6532::peek(uInt16 addr)
       return (myOutA | ~myDDRA) & value;
     }
 
-    case 0x01:    // Port A Data Direction Register 
+    case 0x01:    // SWACNT - Port A Data Direction Register 
     {
       return myDDRA;
     }
 
-    case 0x02:    // Port B I/O Register (Console switches)
+    case 0x02:    // SWCHB - Port B I/O Register (Console switches)
     {
-      return myConsole.switches().read();
+      return (myOutB | ~myDDRB) & (myConsole.switches().read() | myDDRB);
     }
 
-    case 0x03:    // Port B Data Direction Register
+    case 0x03:    // SWBCNT - Port B Data Direction Register
     {
       return myDDRB;
     }
@@ -208,7 +203,7 @@ uInt8 M6532::peek(uInt16 addr)
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void M6532::poke(uInt16 addr, uInt8 value)
+bool M6532::poke(uInt16 addr, uInt8 value)
 {
   // Access RAM directly.  Originally, accesses to RAM could bypass
   // this method and its pages could be installed directly into the
@@ -217,7 +212,7 @@ void M6532::poke(uInt16 addr, uInt8 value)
   if((addr & 0x1080) == 0x0080 && (addr & 0x0200) == 0x0000)
   {
     myRAM[addr & 0x007f] = value;
-    return;
+    return true;
   }
 
   // A2 distinguishes I/O registers from the timer
@@ -233,24 +228,34 @@ void M6532::poke(uInt16 addr, uInt8 value)
   {
     switch(addr & 0x03)
     {
-      case 0:     // Port A I/O Register (Joystick)
+      case 0:     // SWCHA - Port A I/O Register (Joystick)
       {
         myOutA = value;
         setPinState();
         break;
       }
 
-      case 1:     // Port A Data Direction Register 
+      case 1:     // SWACNT - Port A Data Direction Register 
       {
         myDDRA = value;
         setPinState();
         break;
       }
 
-      default:    // Port B I/O & DDR Registers (Console switches)
-        break;    // hardwired as read-only
+      case 2:     // SWCHB - Port B I/O Register (Console switches)
+      {
+        myOutB = value;
+        break;
+      }
+
+      case 3:     // SWBCNT - Port B Data Direction Register 
+      {
+        myDDRB = value;
+        break;
+      }
     }
   }
+  return true;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -296,11 +301,9 @@ void M6532::setPinState()
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 bool M6532::save(Serializer& out) const
 {
-  const string& device = name();
-
   try
   {
-    out.putString(device);
+    out.putString(name());
 
     // Output the RAM
     out.putInt(128);
@@ -316,6 +319,7 @@ bool M6532::save(Serializer& out) const
     out.putByte((char)myDDRA);
     out.putByte((char)myDDRB);
     out.putByte((char)myOutA);
+    out.putByte((char)myOutB);
     out.putByte((char)myOutTimer[0]);
     out.putByte((char)myOutTimer[1]);
     out.putByte((char)myOutTimer[2]);
@@ -333,11 +337,9 @@ bool M6532::save(Serializer& out) const
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 bool M6532::load(Serializer& in)
 {
-  const string& device = name();
-
   try
   {
-    if(in.getString() != device)
+    if(in.getString() != name())
       return false;
 
     // Input the RAM
@@ -354,6 +356,7 @@ bool M6532::load(Serializer& in)
     myDDRA = (uInt8) in.getByte();
     myDDRB = (uInt8) in.getByte();
     myOutA = (uInt8) in.getByte();
+    myOutB = (uInt8) in.getByte();
     myOutTimer[0] = (uInt8) in.getByte();
     myOutTimer[1] = (uInt8) in.getByte();
     myOutTimer[2] = (uInt8) in.getByte();
@@ -370,7 +373,8 @@ bool M6532::load(Serializer& in)
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 M6532::M6532(const M6532& c)
-  : myConsole(c.myConsole)
+  : myConsole(c.myConsole),
+    mySettings(c.mySettings)
 {
   assert(false);
 }
