@@ -8,42 +8,62 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2009 by Bradford W. Mott and the Stella team
+// Copyright (c) 1995-2010 by Bradford W. Mott, Stephen Anthony
+// and the Stella Team
 //
-// See the file "license" for information on usage and redistribution of
+// See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: Serializer.cxx 1859 2009-08-24 23:42:01Z stephena $
+// $Id: Serializer.cxx 2123 2010-09-05 17:57:21Z stephena $
 //============================================================================
 
 #include <fstream>
 #include <sstream>
 
+#include "FSNode.hxx"
 #include "Serializer.hxx"
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-Serializer::Serializer(const string& filename)
+Serializer::Serializer(const string& filename, bool readonly)
   : myStream(NULL),
     myUseFilestream(true)
 {
-  // When using fstreams, we need to manually create the file first
-  // if we want to use it in read/write mode, since it won't be created
-  // if it doesn't already exist
-  // However, if it *does* exist, we don't want to overwrite it
-  // So we open in write and append mode - the write creates the file
-  // when necessary, and the append doesn't delete any data if it
-  // already exists
-  fstream temp(filename.c_str(), ios::out | ios::app);
-  temp.close();
-
-  fstream* str = new fstream(filename.c_str(), ios::in | ios::out | ios::binary);
-  if(str && str->is_open())
+  if(readonly)
   {
-    myStream = str;
-    reset();
+    FilesystemNode node(filename);
+    if(!node.isDirectory() && node.isReadable())
+    {
+      fstream* str = new fstream(filename.c_str(), ios::in | ios::binary);
+      if(str && str->is_open())
+      {
+        myStream = str;
+        reset();
+      }
+      else
+        delete str;
+    }
   }
   else
-    delete str;
+  {
+    // When using fstreams, we need to manually create the file first
+    // if we want to use it in read/write mode, since it won't be created
+    // if it doesn't already exist
+    // However, if it *does* exist, we don't want to overwrite it
+    // So we open in write and append mode - the write creates the file
+    // when necessary, and the append doesn't delete any data if it
+    // already exists
+    fstream temp(filename.c_str(), ios::out | ios::app);
+    temp.close();
+
+    fstream* str = new fstream(filename.c_str(), ios::in | ios::out | ios::binary);
+    if(str && str->is_open())
+    {
+      myStream = str;
+      reset();
+    }
+    else
+      delete str;
+  }
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -132,25 +152,21 @@ string Serializer::getString(void)
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 bool Serializer::getBool(void)
 {
-  bool result = false;
-
   char b = getByte();
   if(b == (char)TruePattern)
-    result = true;
+    return true;
   else if(b == (char)FalsePattern)
-    result = false;
+    return false;
   else
     throw "Serializer::getBool() data corruption";
 
-  return result;
+  return false;  // to stop compiler from complaining
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void Serializer::putByte(char value)
 {
-  char buf[1];
-  buf[0] = value;
-  myStream->write(buf, 1);
+  myStream->write(&value, 1);
   if(myStream->bad())
     throw "Serializer::putByte() file write failed";
 }
