@@ -8,12 +8,13 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2009 by Bradford W. Mott and the Stella team
+// Copyright (c) 1995-2010 by Bradford W. Mott, Stephen Anthony
+// and the Stella Team
 //
-// See the file "license" for information on usage and redistribution of
+// See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: TIA.cxx 1875 2009-09-09 22:08:40Z stephena $
+// $Id: TIA.cxx 2188 2010-11-12 18:54:08Z stephena $
 //============================================================================
 
 #include <cassert>
@@ -22,6 +23,10 @@
 
 #include "bspf.hxx"
 
+#ifdef DEBUGGER_SUPPORT
+  #include "CartDebug.hxx"
+#endif
+
 #include "Console.hxx"
 #include "Control.hxx"
 #include "Device.hxx"
@@ -41,6 +46,9 @@ TIA::TIA(Console& console, Sound& sound, Settings& settings)
   : myConsole(console),
     mySound(sound),
     mySettings(settings),
+    myFrameWidth(160),
+    myFrameYStart(34),
+    myFrameHeight(210),
     myMaximumNumberOfScanlines(262),
     myColorLossEnabled(false),
     myPartialFrameFlag(false),
@@ -125,7 +133,7 @@ void TIA::reset()
 
   // Currently no objects are enabled or selectively disabled
   myEnabledObjects = 0;
-  myDisabledObjects = 0;
+  myDisabledObjects = 0xFF;
   myAllowHMOVEBlanks = true;
 
   // Some default values for the registers
@@ -144,6 +152,7 @@ void TIA::reset()
   myHMP0 = myHMP1 = myHMM0 = myHMM1 = myHMBL = 0;
   myVDELP0 = myVDELP1 = myVDELBL = myRESMP0 = myRESMP1 = false;
   myCollision = 0;
+  myCollisionEnabledMask = 0xFFFFFFFF;
   myPOSP0 = myPOSP1 = myPOSM0 = myPOSM1 = myPOSBL = 0;
 
   // Some default values for the "current" variables
@@ -167,8 +176,10 @@ void TIA::reset()
 
   myDumpEnabled = false;
   myDumpDisabledCycle = 0;
+  myINPT4 = myINPT5 = 0x80;
 
-  myFloatTIAOutputPins = mySettings.getBool("tiafloat");
+  // Should undriven pins be randomly driven high or low?
+  myTIAPinsDriven = mySettings.getBool("tiadriven");
 
   myFrameCounter = 0;
   myScanlineCountForLastFrame = 0;
@@ -202,6 +213,13 @@ void TIA::reset()
     myMaximumNumberOfScanlines = 342;
   }
 
+  myCurrentP0Mask = &TIATables::PxMask[0][0][0][0];
+  myCurrentP1Mask = &TIATables::PxMask[0][0][0][0];
+  myCurrentM0Mask = &TIATables::MxMask[0][0][0][0];
+  myCurrentM1Mask = &TIATables::MxMask[0][0][0][0];
+  myCurrentBLMask = &TIATables::BLMask[0][0][0];
+  myCurrentPFMask = TIATables::PFMask[0];
+
   // Recalculate the size of the display
   frameReset();
 }
@@ -215,15 +233,6 @@ void TIA::frameReset()
   // Reset pixel pointer and drawing flag
   myFramePointer = myCurrentFrameBuffer;
 
-  // Make sure all these are within bounds
-  myFrameWidth  = 160;
-  myFrameYStart = atoi(myConsole.properties().get(Display_YStart).c_str());
-  if(myFrameYStart < 0)       myFrameYStart = 0;
-  else if(myFrameYStart > 64) myFrameYStart = 64;
-  myFrameHeight = atoi(myConsole.properties().get(Display_Height).c_str());
-  if(myFrameHeight < 210)      myFrameHeight = 210;
-  else if(myFrameHeight > 256) myFrameHeight = 256;
-
   // Calculate color clock offsets for starting and stopping frame drawing
   // Note that although we always start drawing at scanline zero, the
   // framebuffer that is exposed outside the class actually starts at 'ystart'
@@ -288,19 +297,12 @@ void TIA::install(System& system, Device& device)
   mySystem->resetCycles();
 
   // All accesses are to the given device
-  System::PageAccess access;
-  access.directPeekBase = 0;
-  access.directPokeBase = 0;
-  access.device = &device;
+  System::PageAccess access(0, 0, 0, &device, System::PA_READWRITE);
 
   // We're installing in a 2600 system
   for(uInt32 i = 0; i < 8192; i += (1 << shift))
-  {
     if((i & 0x1080) == 0x0000)
-    {
       mySystem->setPageAccess(i >> shift, access);
-    }
-  }
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -360,6 +362,7 @@ bool TIA::save(Serializer& out) const
     out.putBool(myRESMP0);
     out.putBool(myRESMP1);
     out.putInt(myCollision);
+    out.putInt(myCollisionEnabledMask);
     out.putByte((char)myCurrentGRP0);
     out.putByte((char)myCurrentGRP1);
 
@@ -468,6 +471,7 @@ bool TIA::load(Serializer& in)
     myRESMP0 = in.getBool();
     myRESMP1 = in.getBool();
     myCollision = (uInt16) in.getInt();
+    myCollisionEnabledMask = in.getInt();
     myCurrentGRP0 = (uInt8) in.getByte();
     myCurrentGRP1 = (uInt8) in.getByte();
 
@@ -742,6 +746,49 @@ bool TIA::toggleBit(TIABit b, uInt8 mode)
   return on;
 }
 
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void TIA::enableCollisions(bool mode)
+{
+  toggleCollision(P0Bit, mode ? 1 : 0);
+  toggleCollision(P1Bit, mode ? 1 : 0);
+  toggleCollision(M0Bit, mode ? 1 : 0);
+  toggleCollision(M1Bit, mode ? 1 : 0);
+  toggleCollision(BLBit, mode ? 1 : 0);
+  toggleCollision(PFBit, mode ? 1 : 0);
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+bool TIA::toggleCollision(TIABit b, uInt8 mode)
+{
+  uInt16 enabled = myCollisionEnabledMask >> 16;
+
+  // If mode is 0 or 1, use it as a boolean (off or on)
+  // Otherwise, flip the state
+  bool on = (mode == 0 || mode == 1) ? bool(mode) : !(enabled & b);
+  if(on)  enabled |= b;
+  else    enabled &= ~b;
+
+  // Assume all collisions are on, then selectively turn the desired ones off
+  uInt16 mask = 0xffff;
+  if(!(enabled & P0Bit))
+    mask &= ~(Cx_M0P0 | Cx_M1P0 | Cx_P0PF | Cx_P0BL | Cx_P0P1);
+  if(!(enabled & P1Bit))
+    mask &= ~(Cx_M0P1 | Cx_M1P1 | Cx_P1PF | Cx_P1BL | Cx_P0P1);
+  if(!(enabled & M0Bit))
+    mask &= ~(Cx_M0P0 | Cx_M0P1 | Cx_M0PF | Cx_M0BL | Cx_M0M1);
+  if(!(enabled & M1Bit))
+    mask &= ~(Cx_M1P0 | Cx_M1P1 | Cx_M1PF | Cx_M1BL | Cx_M0M1);
+  if(!(enabled & BLBit))
+    mask &= ~(Cx_P0BL | Cx_P1BL | Cx_M0BL | Cx_M1BL | Cx_BLPF);
+  if(!(enabled & PFBit))
+    mask &= ~(Cx_P0PF | Cx_P1PF | Cx_M0PF | Cx_M1PF | Cx_BLPF);
+
+  // Now combine the masks
+  myCollisionEnabledMask = (enabled << 16) | mask;
+
+  return on;
+}
+
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 bool TIA::toggleHMOVEBlank()
 {
@@ -965,22 +1012,48 @@ void TIA::updateFrame(Int32 clock)
         // simulates the behaviour as visually seen in the aforementioned
         // ROMs.  Other ROMs may break this simulation; more testing is
         // required to figure out what's really going on here.
-        if(myHMM0mmr && myPOSM0 % 4 == 3)
+        if(myHMM0mmr)
         {
-          // Stretch this missle so it's 4 pixels wide, with the 3rd pixel
-          // blanked out; this is indicated by using a size of '4'
-          myCurrentM0Mask = &TIATables::MxMask[myPOSM0 & 0x03]
-              [myNUSIZ0 & 0x07][4][160 - (myPOSM0 & 0xFC)];
+          switch(myPOSM0 % 4)
+          {
+            case 3:
+              // Stretch this missle so it's 2 pixels wide and shifted one
+              // pixel to the left
+              myCurrentM0Mask = &TIATables::MxMask[(myPOSM0-1) & 0x03]
+                  [myNUSIZ0 & 0x07][((myNUSIZ0 & 0x30) >> 4)|1][160 - ((myPOSM0-1) & 0xFC)];
+              break;
+            case 2:
+              // Missle is disabled on this line
+              myCurrentM0Mask = &TIATables::DisabledMask[0];
+              break;
+            default:
+              myCurrentM0Mask = &TIATables::MxMask[myPOSM0 & 0x03]
+                  [myNUSIZ0 & 0x07][(myNUSIZ0 & 0x30) >> 4][160 - (myPOSM0 & 0xFC)];
+              break;
+          }
         }
         else
           myCurrentM0Mask = &TIATables::MxMask[myPOSM0 & 0x03]
               [myNUSIZ0 & 0x07][(myNUSIZ0 & 0x30) >> 4][160 - (myPOSM0 & 0xFC)];
-        if(myHMM1mmr && myPOSM1 % 4 == 3)
+        if(myHMM1mmr)
         {
-          // Stretch this missle so it's 4 pixels wide, with the 3rd pixel
-          // blanked out; this is indicated by using a size of '4'
-          myCurrentM1Mask = &TIATables::MxMask[myPOSM1 & 0x03]
-              [myNUSIZ1 & 0x07][4][160 - (myPOSM1 & 0xFC)];
+          switch(myPOSM1 % 4)
+          {
+            case 3:
+              // Stretch this missle so it's 2 pixels wide and shifted one
+              // pixel to the left
+              myCurrentM1Mask = &TIATables::MxMask[(myPOSM1-1) & 0x03]
+                  [myNUSIZ1 & 0x07][((myNUSIZ1 & 0x30) >> 4)|1][160 - ((myPOSM1-1) & 0xFC)];
+              break;
+            case 2:
+              // Missle is disabled on this line
+              myCurrentM1Mask = &TIATables::DisabledMask[0];
+              break;
+            default:
+              myCurrentM1Mask = &TIATables::MxMask[myPOSM1 & 0x03]
+                  [myNUSIZ1 & 0x07][(myNUSIZ1 & 0x30) >> 4][160 - (myPOSM1 & 0xFC)];
+              break;
+          }
         }
         else
           myCurrentM1Mask = &TIATables::MxMask[myPOSM1 & 0x03]
@@ -1088,87 +1161,103 @@ uInt8 TIA::peek(uInt16 addr)
   // Update frame to current color clock before we look at anything!
   updateFrame(mySystem->cycles() * 3);
 
-  uInt8 value = 0x00;
+  // If pins are undriven, we start with the last databus value
+  // Otherwise, there is some randomness injected into the mix
+  // In either case, we start out with D7 and D6 disabled (the only
+  // valid bits in a TIA read), and selectively enable them
+  uInt8 value = 0x3F & (!myTIAPinsDriven ? mySystem->getDataBusState() :
+                        mySystem->getDataBusState(0xFF));
+  uInt16 collision = myCollision & (uInt16)myCollisionEnabledMask;
 
   switch(addr & 0x000f)
   {
     case CXM0P:
-      value = ((myCollision & Cx_M0P1) ? 0x80 : 0x00) |
-              ((myCollision & Cx_M0P0) ? 0x40 : 0x00);
+      value |= ((collision & Cx_M0P1) ? 0x80 : 0x00) |
+               ((collision & Cx_M0P0) ? 0x40 : 0x00);
       break;
 
     case CXM1P:
-      value = ((myCollision & Cx_M1P0) ? 0x80 : 0x00) |
-              ((myCollision & Cx_M1P1) ? 0x40 : 0x00);
+      value |= ((collision & Cx_M1P0) ? 0x80 : 0x00) |
+               ((collision & Cx_M1P1) ? 0x40 : 0x00);
       break;
 
     case CXP0FB:
-      value = ((myCollision & Cx_P0PF) ? 0x80 : 0x00) |
-              ((myCollision & Cx_P0BL) ? 0x40 : 0x00);
+      value |= ((collision & Cx_P0PF) ? 0x80 : 0x00) |
+               ((collision & Cx_P0BL) ? 0x40 : 0x00);
       break;
 
     case CXP1FB:
-      value = ((myCollision & Cx_P1PF) ? 0x80 : 0x00) |
-              ((myCollision & Cx_P1BL) ? 0x40 : 0x00);
+      value |= ((collision & Cx_P1PF) ? 0x80 : 0x00) |
+               ((collision & Cx_P1BL) ? 0x40 : 0x00);
       break;
 
     case CXM0FB:
-      value = ((myCollision & Cx_M0PF) ? 0x80 : 0x00) |
-              ((myCollision & Cx_M0BL) ? 0x40 : 0x00);
+      value |= ((collision & Cx_M0PF) ? 0x80 : 0x00) |
+               ((collision & Cx_M0BL) ? 0x40 : 0x00);
       break;
 
     case CXM1FB:
-      value = ((myCollision & Cx_M1PF) ? 0x80 : 0x00) |
-              ((myCollision & Cx_M1BL) ? 0x40 : 0x00);
+      value |= ((collision & Cx_M1PF) ? 0x80 : 0x00) |
+               ((collision & Cx_M1BL) ? 0x40 : 0x00);
       break;
 
     case CXBLPF:
-      value = (myCollision & Cx_BLPF) ? 0x80 : 0x00;
+      value = (value & 0x7F) | ((collision & Cx_BLPF) ? 0x80 : 0x00);
       break;
 
     case CXPPMM:
-      value = ((myCollision & Cx_P0P1) ? 0x80 : 0x00) |
-              ((myCollision & Cx_M0M1) ? 0x40 : 0x00);
+      value |= ((collision & Cx_P0P1) ? 0x80 : 0x00) |
+               ((collision & Cx_M0M1) ? 0x40 : 0x00);
       break;
 
     case INPT0:
-      value = dumpedInputPort(myConsole.controller(Controller::Left).read(Controller::Nine));
+      value = (value & 0x7F) |
+        dumpedInputPort(myConsole.controller(Controller::Left).read(Controller::Nine));
       break;
 
     case INPT1:
-      value = dumpedInputPort(myConsole.controller(Controller::Left).read(Controller::Five));
+      value = (value & 0x7F) |
+        dumpedInputPort(myConsole.controller(Controller::Left).read(Controller::Five));
       break;
 
     case INPT2:
-      value = dumpedInputPort(myConsole.controller(Controller::Right).read(Controller::Nine));
+      value = (value & 0x7F) |
+        dumpedInputPort(myConsole.controller(Controller::Right).read(Controller::Nine));
       break;
 
     case INPT3:
-      value = dumpedInputPort(myConsole.controller(Controller::Right).read(Controller::Five));
+      value = (value & 0x7F) |
+        dumpedInputPort(myConsole.controller(Controller::Right).read(Controller::Five));
       break;
 
     case INPT4:
-      value = myConsole.controller(Controller::Left).read(Controller::Six) ? 0x80 : 0x00;
+    {
+      uInt8 button = (myConsole.controller(Controller::Left).read(Controller::Six) ? 0x80 : 0x00);
+      myINPT4 = (myVBLANK & 0x40) ? (myINPT4 & button) : button;
+
+      value = (value & 0x7F) | myINPT4;
       break;
+    }
 
     case INPT5:
-      value = myConsole.controller(Controller::Right).read(Controller::Six) ? 0x80 : 0x00;
+    {
+      uInt8 button = (myConsole.controller(Controller::Right).read(Controller::Six) ? 0x80 : 0x00);
+      myINPT5 = (myVBLANK & 0x40) ? (myINPT5 & button) : button;
+
+      value = (value & 0x7F) | myINPT5;
       break;
+    }
 
     default:
+      // This shouldn't happen, but if it does, we essentially just
+      // return the last databus value with bits D6 and D7 zeroed out
       break;
   }
-
-  // On certain CMOS EPROM chips the unused TIA pins on a read are not
-  // floating but pulled high. Programmers might want to check their
-  // games for compatibility, so we make this optional. 
-  value |= myFloatTIAOutputPins ? (mySystem->getDataBusState() & 0x3F) : 0x3F;
-
   return value;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void TIA::poke(uInt16 addr, uInt8 value)
+bool TIA::poke(uInt16 addr, uInt8 value)
 {
   addr = addr & 0x003f;
 
@@ -1232,6 +1321,10 @@ void TIA::poke(uInt16 addr, uInt8 value)
         myDumpDisabledCycle = mySystem->cycles();
       }
 
+      // Are the latches for I4 and I5 being set?
+      if (!(myVBLANK & 0x40))
+        myINPT4 = myINPT5 = 0x80;
+
 #if 0 // TODO - this isn't yet complete
       // Check for the first scanline at which VBLANK is disabled.
       // Usually, this will be the first scanline to start drawing.
@@ -1345,9 +1438,6 @@ void TIA::poke(uInt16 addr, uInt8 value)
       if(((clock - myClockWhenFrameStarted) % 228) < (68 + 79))
         myCurrentPFMask = TIATables::PFMask[myCTRLPF & 0x01];
 
-      myCurrentBLMask = &TIATables::BLMask[myPOSBL & 0x03]
-          [(myCTRLPF & 0x30) >> 4][160 - (myPOSBL & 0xFC)];
-
       break;
     }
 
@@ -1382,6 +1472,11 @@ void TIA::poke(uInt16 addr, uInt8 value)
       else
         myEnabledObjects |= PFBit;
 
+    #ifdef DEBUGGER_SUPPORT
+      uInt16 dataAddr = mySystem->m6502().lastDataAddressForPoke();
+      if(dataAddr)
+        mySystem->setAccessFlags(dataAddr, CartDebug::PGFX);
+    #endif
       break;
     }
 
@@ -1394,6 +1489,11 @@ void TIA::poke(uInt16 addr, uInt8 value)
       else
         myEnabledObjects |= PFBit;
 
+    #ifdef DEBUGGER_SUPPORT
+      uInt16 dataAddr = mySystem->m6502().lastDataAddressForPoke();
+      if(dataAddr)
+        mySystem->setAccessFlags(dataAddr, CartDebug::PGFX);
+    #endif
       break;
     }
 
@@ -1406,6 +1506,11 @@ void TIA::poke(uInt16 addr, uInt8 value)
       else
         myEnabledObjects |= PFBit;
 
+    #ifdef DEBUGGER_SUPPORT
+      uInt16 dataAddr = mySystem->m6502().lastDataAddressForPoke();
+      if(dataAddr)
+        mySystem->setAccessFlags(dataAddr, CartDebug::PGFX);
+    #endif
       break;
     }
 
@@ -1641,6 +1746,11 @@ void TIA::poke(uInt16 addr, uInt8 value)
       else
         myEnabledObjects &= ~P1Bit;
 
+    #ifdef DEBUGGER_SUPPORT
+      uInt16 dataAddr = mySystem->m6502().lastDataAddressForPoke();
+      if(dataAddr)
+        mySystem->setAccessFlags(dataAddr, CartDebug::GFX);
+    #endif
       break;
     }
 
@@ -1679,6 +1789,11 @@ void TIA::poke(uInt16 addr, uInt8 value)
       else
         myEnabledObjects &= ~BLBit;
 
+    #ifdef DEBUGGER_SUPPORT
+      uInt16 dataAddr = mySystem->m6502().lastDataAddressForPoke();
+      if(dataAddr)
+        mySystem->setAccessFlags(dataAddr, CartDebug::GFX);
+    #endif
       break;
     }
 
@@ -1955,6 +2070,7 @@ void TIA::poke(uInt16 addr, uInt8 value)
       break;
     }
   }
+  return true;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
