@@ -8,12 +8,13 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2009 by Bradford W. Mott and the Stella team
+// Copyright (c) 1995-2010 by Bradford W. Mott, Stephen Anthony
+// and the Stella Team
 //
-// See the file "license" for information on usage and redistribution of
+// See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: GameInfoDialog.cxx 1842 2009-07-27 17:17:07Z stephena $
+// $Id: GameInfoDialog.cxx 2097 2010-08-14 01:01:27Z stephena $
 //
 //   Based on code from ScummVM - Scumm Interpreter
 //   Copyright (C) 2002-2004 The ScummVM project
@@ -42,6 +43,7 @@ GameInfoDialog::GameInfoDialog(
     myPropertiesLoaded(false),
     myDefaultsSelected(false)
 {
+  const GUI::Font& ifont = instance().infoFont();
   const int lineHeight   = font.getLineHeight(),
             fontWidth    = font.getMaxCharWidth(),
             fontHeight   = font.getFontHeight(),
@@ -65,7 +67,7 @@ GameInfoDialog::GameInfoDialog(
   // The tab widget
   xpos = 2; ypos = vBorder;
   myTab = new TabWidget(this, font, xpos, ypos, _w - 2*xpos,
-                        _h - buttonHeight - 2*fontHeight - 20);
+                        _h - buttonHeight - fontHeight - ifont.getLineHeight() - 20);
   addTabWidget(myTab);
   addFocusWidget(myTab);
 
@@ -138,6 +140,7 @@ GameInfoDialog::GameInfoDialog(
   items.push_back("2IN1 Multicart (4-32K)",     "2IN1" );
   items.push_back("4IN1 Multicart (8-32K)",     "4IN1" );
   items.push_back("8IN1 Multicart (16-64K)",    "8IN1" );
+  items.push_back("16IN1 Multicart (32-128K)",  "16IN1");
   items.push_back("32IN1 Multicart (64-128K)",  "32IN1");
   items.push_back("2K (64-2048 bytes Atari)",   "2K"   );
   items.push_back("3E (32K Tigervision)",       "3E"   );
@@ -147,6 +150,7 @@ GameInfoDialog::GameInfoDialog(
   items.push_back("AR (Supercharger)",          "AR"   );
   items.push_back("CV (Commavid extra ram)",    "CV"   );
   items.push_back("DPC (Pitfall II)",           "DPC"  );
+  items.push_back("DPC+ (Enhanced DPC)",        "DPC+" );
   items.push_back("E0 (8K Parker Bros)",        "E0"   );
   items.push_back("E7 (16K M-network)",         "E7"   );
   items.push_back("EF (64K H. Runner)",         "EF"   );
@@ -201,7 +205,7 @@ GameInfoDialog::GameInfoDialog(
                        "TV Type:", kTextAlignLeft);
   items.clear();
   items.push_back("Color", "COLOR");
-  items.push_back("B & W", "BLACKANDWHITE");
+  items.push_back("B & W", "BW");
   myTVType = new PopUpWidget(myTab, font, xpos+lwidth, ypos,
                              pwidth, lineHeight, items, "", 0, 0);
   wid.push_back(myTVType);
@@ -230,7 +234,8 @@ GameInfoDialog::GameInfoDialog(
   ctrls.push_back("AmigaMouse",     "AMIGAMOUSE"  );
   ctrls.push_back("AtariVox",       "ATARIVOX"    );
   ctrls.push_back("SaveKey",        "SAVEKEY"     );
-//FIXME  ctrls.push_back("KidVid",         "KIDVID"      );
+  ctrls.push_back("Sega Genesis",   "GENESIS"     );
+// TODO  ctrls.push_back("KidVid",         "KIDVID"      );
   myP0Controller = new PopUpWidget(myTab, font, xpos+lwidth, ypos,
                                    pwidth, lineHeight, ctrls, "", 0, 0);
   wid.push_back(myP0Controller);
@@ -347,8 +352,8 @@ GameInfoDialog::GameInfoDialog(
   myTab->setActiveTab(0);
 
   // Add message concerning usage
-  lwidth = font.getStringWidth("(*) Changes to properties require a ROM reload");
-  new StaticTextWidget(this, font, 10, _h - buttonHeight - fontHeight - 20,
+  lwidth = ifont.getStringWidth("(*) Changes to properties require a ROM reload");
+  new StaticTextWidget(this, ifont, 10, _h - buttonHeight - fontHeight - 20,
                        lwidth, fontHeight,
                        "(*) Changes to properties require a ROM reload",
                        kTextAlignLeft);
