@@ -8,12 +8,13 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2009 by Bradford W. Mott and the Stella team
+// Copyright (c) 1995-2010 by Bradford W. Mott, Stephen Anthony
+// and the Stella Team
 //
-// See the file "license" for information on usage and redistribution of
+// See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: LauncherDialog.cxx 1873 2009-09-09 15:59:22Z stephena $
+// $Id: LauncherDialog.cxx 2076 2010-07-29 15:22:27Z stephena $
 //
 //   Based on code from ScummVM - Scumm Interpreter
 //   Copyright (C) 2002-2004 The ScummVM project
@@ -34,6 +35,7 @@
 #include "OptionsDialog.hxx"
 #include "GlobalPropsDialog.hxx"
 #include "LauncherFilterDialog.hxx"
+#include "MessageBox.hxx"
 #include "OSystem.hxx"
 #include "Props.hxx"
 #include "PropsSet.hxx"
@@ -60,6 +62,8 @@ LauncherDialog::LauncherDialog(OSystem* osystem, DialogContainer* parent,
     myMenu(NULL),
     myGlobalProps(NULL),
     myFilters(NULL),
+    myFirstRunMsg(NULL),
+    myRomDir(NULL),
     mySelectedItem(0)
 {
   const GUI::Font& font = instance().launcherFont();
@@ -85,10 +89,13 @@ LauncherDialog::LauncherDialog(OSystem* osystem, DialogContainer* parent,
 
   // Add filter that can narrow the results shown in the listing
   // It has to fit between both labels
-  fwidth = BSPF_min(15 * fontWidth, xpos - 20 - lwidth);
-  xpos -= fwidth + 5;
-  myPattern = new EditTextWidget(this, font, xpos, ypos,
-                                 fwidth, fontHeight, "");
+  if(w >= 640)
+  {
+    fwidth = BSPF_min(15 * fontWidth, xpos - 20 - lwidth);
+    xpos -= fwidth + 5;
+    myPattern = new EditTextWidget(this, font, xpos, ypos,
+                                   fwidth, fontHeight, "");
+  }
 
   // Add list with game titles
   // Before we add the list, we need to know the size of the RomInfoWidget
@@ -106,7 +113,7 @@ LauncherDialog::LauncherDialog(OSystem* osystem, DialogContainer* parent,
   myList->setNumberingMode(kListNumberingOff);
   myList->setEditable(false);
   wid.push_back(myList);
-  wid.push_back(myPattern);  // Add after the list for tab order
+  if(myPattern)  wid.push_back(myPattern);  // Add after the list for tab order
 
   // Add ROM info area (if enabled)
   if(romWidth > 0)
@@ -168,7 +175,7 @@ LauncherDialog::LauncherDialog(OSystem* osystem, DialogContainer* parent,
   mySelectedItem = 0;  // Highlight 'Rom Listing'
 
   // Create an options dialog, similar to the in-game one
-  myOptions = new OptionsDialog(osystem, parent, this, true);  // not in game mode
+  myOptions = new OptionsDialog(osystem, parent, this, w, h, true);  // not in game mode
 
   // Create a game list, which contains all the information about a ROM that
   // the launcher needs
@@ -187,8 +194,7 @@ LauncherDialog::LauncherDialog(OSystem* osystem, DialogContainer* parent,
   // ROM properties
   myGlobalProps = new GlobalPropsDialog(this, osystem->font());
 
-  // Create dialog whereby the files shown in the ROM listing can be
-  // customized
+  // Create dialog whereby the files shown in the ROM listing can be customized
   myFilters = new LauncherFilterDialog(this, osystem->font());
 
   // Figure out which filters are needed for the ROM listing
@@ -203,6 +209,7 @@ LauncherDialog::~LauncherDialog()
   delete myMenu;
   delete myGlobalProps;
   delete myFilters;
+  delete myRomDir;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -226,12 +233,33 @@ string LauncherDialog::selectedRomMD5()
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void LauncherDialog::loadConfig()
 {
+  const string& romdir = instance().settings().getString("romdir");
+
+  // When romdir hasn't been set, it probably indicates that this is the first
+  // time running Stella; in this case, we should prompt the user
+  if(romdir == "")
+  {
+    if(!myFirstRunMsg)
+    {
+      StringList msg;
+      msg.push_back("This seems to be your first time running Stella.");
+      msg.push_back("Before you can start a game, you need to");
+      msg.push_back("specify where your ROMs are located.");
+      msg.push_back("");
+      msg.push_back("Click 'OK' to select a default ROM directory,");
+      msg.push_back("or 'Cancel' to browse the filesystem manually.");
+      myFirstRunMsg = new MessageBox(this, instance().font(), msg,
+                                     _w, _h, kFirstRunMsgChosenCmd);
+    }
+    myFirstRunMsg->show();
+  }
+
   // Assume that if the list is empty, this is the first time that loadConfig()
-  // has been called (and we should reload the list).
+  // has been called (and we should reload the list)
   if(myList->getList().isEmpty())
   {
     myPrevDirButton->setEnabled(false);
-    myCurrentNode = FilesystemNode(instance().settings().getString("romdir"));
+    myCurrentNode = FilesystemNode(romdir == "" ? "~" : romdir);
     if(!(myCurrentNode.exists() && myCurrentNode.isDirectory()))
       myCurrentNode = FilesystemNode("~");
 
@@ -259,7 +287,6 @@ void LauncherDialog::updateListing(const string& nameToSelect)
   myGameList->clear();
   myDir->setLabel("");
 
-  string romdir = instance().settings().getString("romdir");
   loadDirListing();
 
   // Only hilite the 'up' button if there's a parent directory
@@ -323,6 +350,7 @@ void LauncherDialog::loadDirListing()
     myGameList->appendGame(" [..]", "", "", true);
 
   // Now add the directory entries
+  bool domatch = myPattern && myPattern->getEditString() != "";
   for(unsigned int idx = 0; idx < files.size(); idx++)
   {
     string name = files[idx].getDisplayName();
@@ -342,8 +370,7 @@ void LauncherDialog::loadDirListing()
     }
 
     // Skip over files that don't match the pattern in the 'pattern' textbox
-    if(!isDir && myPattern->getEditString() != "" &&
-       !matchPattern(name, myPattern->getEditString()))
+    if(domatch && !isDir && !matchPattern(name, myPattern->getEditString()))
       continue;
 
     myGameList->appendGame(name, files[idx].getPath(), "", isDir);
@@ -499,7 +526,7 @@ void LauncherDialog::handleCommand(CommandSender* sender, int cmd,
             if(!myNodeNames.empty())
               dirname = myNodeNames.pop();
           }
-		  else
+          else
           {
             myCurrentNode = FilesystemNode(rom);
             myNodeNames.push(myGameList->name(item));
@@ -518,10 +545,11 @@ void LauncherDialog::handleCommand(CommandSender* sender, int cmd,
             }
             else
               instance().frameBuffer().showMessage(
-                  "Error creating console (screen too small)", kMiddleCenter);
+                  "Error creating console (screen too small)", kMiddleCenter, true);
           }
           else
-            instance().frameBuffer().showMessage("Not a valid ROM file", kMiddleCenter);
+            instance().frameBuffer().showMessage("Not a valid ROM file",
+                                                 kMiddleCenter, true);
         }
       }
       break;
@@ -546,6 +574,21 @@ void LauncherDialog::handleCommand(CommandSender* sender, int cmd,
       instance().eventHandler().quit();
       break;
 
+    case kFirstRunMsgChosenCmd:
+      // Show a file browser, starting from the users' home directory
+      if(!myRomDir)
+        myRomDir = new BrowserDialog(this, instance().font(), _w, _h);
+
+      myRomDir->show("Select ROM directory:", "~",
+                     FilesystemNode::kListDirectoriesOnly, kStartupRomDirChosenCmd);
+      break;
+
+    case kStartupRomDirChosenCmd:
+    {
+      FilesystemNode dir(myRomDir->getResult());
+      instance().settings().setString("romdir", dir.getRelativePath());
+      // fall through to the next case
+    }
     case kRomDirChosenCmd:
       myCurrentNode = FilesystemNode(instance().settings().getString("romdir"));
       if(!(myCurrentNode.exists() && myCurrentNode.isDirectory()))
