@@ -8,12 +8,13 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2009 by Bradford W. Mott and the Stella team
+// Copyright (c) 1995-2010 by Bradford W. Mott, Stephen Anthony
+// and the Stella Team
 //
-// See the file "license" for information on usage and redistribution of
+// See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: UIDialog.cxx 1775 2009-06-17 17:04:55Z stephena $
+// $Id: UIDialog.cxx 2097 2010-08-14 01:01:27Z stephena $
 //
 //   Based on code from ScummVM - Scumm Interpreter
 //   Copyright (C) 2002-2004 The ScummVM project
@@ -40,6 +41,7 @@ UIDialog::UIDialog(OSystem* osystem, DialogContainer* parent,
                    const GUI::Font& font)
   : Dialog(osystem, parent, 0, 0, 0, 0)
 {
+  const GUI::Font& ifont = instance().infoFont();
   const int lineHeight   = font.getLineHeight(),
             fontWidth    = font.getMaxCharWidth(),
             fontHeight   = font.getFontHeight(),
@@ -52,8 +54,8 @@ UIDialog::UIDialog(OSystem* osystem, DialogContainer* parent,
   StringMap items;
 
   // Set real dimensions
-  _w = 42 * fontWidth + 10;
-  _h = 10 * (lineHeight + 4) + 10;
+  _w = 37 * fontWidth + 10;
+  _h = 9 * (lineHeight + 4) + 10;
 
   // The tab widget
   xpos = ypos = vBorder;
@@ -71,8 +73,16 @@ UIDialog::UIDialog(OSystem* osystem, DialogContainer* parent,
   myLauncherWidthSlider = new SliderWidget(myTab, font, xpos, ypos, pwidth,
                                            lineHeight, "Launcher Width: ",
                                            lwidth, kLWidthChanged);
-  myLauncherWidthSlider->setMinValue(320);
-  myLauncherWidthSlider->setMaxValue(1920);
+  if(instance().desktopWidth() < 640)
+  {
+    myLauncherWidthSlider->setMinValue(320);
+    myLauncherWidthSlider->setMaxValue(instance().desktopWidth());
+  }
+  else
+  {
+    myLauncherWidthSlider->setMinValue(640);
+    myLauncherWidthSlider->setMaxValue(1920);
+  }
   myLauncherWidthSlider->setStepValue(10);
   wid.push_back(myLauncherWidthSlider);
   myLauncherWidthLabel =
@@ -85,8 +95,16 @@ UIDialog::UIDialog(OSystem* osystem, DialogContainer* parent,
   myLauncherHeightSlider = new SliderWidget(myTab, font, xpos, ypos, pwidth,
                                             lineHeight, "Launcher Height: ",
                                             lwidth, kLHeightChanged);
-  myLauncherHeightSlider->setMinValue(240);
-  myLauncherHeightSlider->setMaxValue(1200);
+  if(instance().desktopHeight() < 480)
+  {
+    myLauncherHeightSlider->setMinValue(240);
+    myLauncherHeightSlider->setMaxValue(instance().desktopHeight());
+  }
+  else
+  {
+    myLauncherHeightSlider->setMinValue(480);
+    myLauncherHeightSlider->setMaxValue(1200);
+  }
   myLauncherHeightSlider->setStepValue(10);
   wid.push_back(myLauncherHeightSlider);
   myLauncherHeightLabel =
@@ -119,15 +137,10 @@ UIDialog::UIDialog(OSystem* osystem, DialogContainer* parent,
   wid.push_back(myRomViewerPopup);
   ypos += lineHeight + 4;
 
-  // Should we use the built-in ROM launcher
-  myUseLauncherBox = new CheckboxWidget(myTab, font, xpos+20, ypos+10,
-                                        "Use ROM launcher");
-  wid.push_back(myUseLauncherBox);
-
   // Add message concerning usage
-  xpos = vBorder; ypos += 2*(lineHeight + 4);
-  lwidth = font.getStringWidth("(*) Changes require application restart");
-  new StaticTextWidget(myTab, font, xpos, ypos, lwidth, fontHeight,
+  xpos = vBorder; ypos += 1*(lineHeight + 4);
+  lwidth = ifont.getStringWidth("(*) Changes require application restart");
+  new StaticTextWidget(myTab, ifont, xpos, ypos, BSPF_min(lwidth, _w-20), fontHeight,
                        "(*) Changes require application restart",
                        kTextAlignLeft);
 
@@ -170,12 +183,21 @@ UIDialog::UIDialog(OSystem* osystem, DialogContainer* parent,
                            ypos + 1, 4*fontWidth, fontHeight, "", kTextAlignLeft);
   myDebuggerHeightLabel->setFlags(WIDGET_CLEARBG);
 
-  // Add message concerning usage
-  xpos = vBorder; ypos += 2*(lineHeight + 4);
-  lwidth = font.getStringWidth("(*) Changes require application restart");
-  new StaticTextWidget(myTab, font, xpos, ypos, lwidth, fontHeight,
-                       "(*) Changes require application restart",
-                       kTextAlignLeft);
+  // Debugger is only realistically available in windowed modes 800x600 or greater
+  // (and when it's actually been compiled into the app)
+  bool debuggerAvailable = 
+#if defined(DEBUGGER_SUPPORT) && defined(WINDOWED_SUPPORT)
+    (instance().desktopWidth() >= 800 && instance().desktopHeight() >= 600);
+#else
+  false;
+#endif
+  if(!debuggerAvailable)
+  {
+    myDebuggerWidthSlider->clearFlags(WIDGET_ENABLED);
+    myDebuggerWidthLabel->clearFlags(WIDGET_ENABLED);
+    myDebuggerHeightSlider->clearFlags(WIDGET_ENABLED);
+    myDebuggerHeightLabel->clearFlags(WIDGET_ENABLED);
+  }
 
   // Add items for tab 1
   addToFocusList(wid, tabID);
@@ -198,33 +220,45 @@ UIDialog::UIDialog(OSystem* osystem, DialogContainer* parent,
   ypos += lineHeight + 4;
 
   // Delay between quick-selecting characters in ListWidget
-  myListDelaySlider = new SliderWidget(myTab, font, xpos, ypos, pwidth,
-                                       lineHeight, "List quick delay: ",
-                                       lwidth, kLQDelayChanged);
-  myListDelaySlider->setMinValue(300);
-  myListDelaySlider->setMaxValue(1000);
-  myListDelaySlider->setStepValue(100);
-  wid.push_back(myListDelaySlider);
-  myListDelayLabel =
-      new StaticTextWidget(myTab, font,
-                           xpos + myListDelaySlider->getWidth() + 4,
-                           ypos + 1, 4*fontWidth, fontHeight, "", kTextAlignLeft);
-  myListDelayLabel->setFlags(WIDGET_CLEARBG);
+  items.clear();
+  items.push_back("300 ms", "300");
+  items.push_back("400 ms", "400");
+  items.push_back("500 ms", "500");
+  items.push_back("600 ms", "600");
+  items.push_back("700 ms", "700");
+  items.push_back("800 ms", "800");
+  items.push_back("900 ms", "900");
+  items.push_back("1 sec", "1000");
+  myListDelayPopup = new PopUpWidget(myTab, font, xpos, ypos, pwidth, lineHeight,
+                                     items, "List quick delay: ", lwidth);
+  wid.push_back(myListDelayPopup);
   ypos += lineHeight + 4;
 
   // Number of lines a mouse wheel will scroll
-  myWheelLinesSlider = new SliderWidget(myTab, font, xpos, ypos, pwidth,
-                                        lineHeight, "Mouse wheel scroll: ",
-                                        lwidth, kWLinesChanged);
-  myWheelLinesSlider->setMinValue(1);
-  myWheelLinesSlider->setMaxValue(10);
-  myWheelLinesSlider->setStepValue(1);
-  wid.push_back(myWheelLinesSlider);
-  myWheelLinesLabel =
-      new StaticTextWidget(myTab, font,
-                           xpos + myWheelLinesSlider->getWidth() + 4,
-                           ypos + 1, 2*fontWidth, fontHeight, "", kTextAlignLeft);
-  myWheelLinesLabel->setFlags(WIDGET_CLEARBG);
+  items.clear();
+  items.push_back("1 line", "1");
+  items.push_back("2 lines", "2");
+  items.push_back("3 lines", "3");
+  items.push_back("4 lines", "4");
+  items.push_back("5 lines", "5");
+  items.push_back("6 lines", "6");
+  items.push_back("7 lines", "7");
+  items.push_back("8 lines", "8");
+  items.push_back("9 lines", "9");
+  items.push_back("10 lines", "10");
+  myWheelLinesPopup = new PopUpWidget(myTab, font, xpos, ypos, pwidth, lineHeight,
+                                      items, "Mouse wheel scroll: ", lwidth);
+  wid.push_back(myWheelLinesPopup);
+  ypos += lineHeight + 4;
+
+  // Amount of output to show with 'showinfo'
+  items.clear();
+  items.push_back("None", "0");
+  items.push_back("Basic", "1");
+  items.push_back("Verbose", "2");
+  myShowInfoPopup = new PopUpWidget(myTab, font, xpos, ypos, pwidth, lineHeight,
+                                    items, "Show Info level: ", lwidth);
+  wid.push_back(myShowInfoPopup);
   ypos += lineHeight + 4;
 
   // Add items for tab 2
@@ -241,17 +275,6 @@ UIDialog::UIDialog(OSystem* osystem, DialogContainer* parent,
   wid.push_back(b);
   addOKCancelBGroup(wid, font);
   addBGroupToFocusList(wid);
-
-#ifndef DEBUGGER_SUPPORT
-  myDebuggerWidthSlider->clearFlags(WIDGET_ENABLED);
-  myDebuggerWidthLabel->clearFlags(WIDGET_ENABLED);
-  myDebuggerHeightSlider->clearFlags(WIDGET_ENABLED);
-  myDebuggerHeightLabel->clearFlags(WIDGET_ENABLED);
-#endif
-
-#ifdef _WIN32_WCE
-  myLauncherPopup->clearFlags(WIDGET_ENABLED);
-#endif
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -284,9 +307,7 @@ void UIDialog::loadConfig()
   const string& viewer = instance().settings().getString("romviewer");
   myRomViewerPopup->setSelected(viewer, "0");
 
-  // Use ROM launcher
-  myUseLauncherBox->setState(instance().settings().getBool("uselauncher"));
-
+#ifdef DEBUGGER_SUPPORT
   // Debugger size
   instance().settings().getSize("debuggerres", w, h);
   w = BSPF_max(w, 1050);
@@ -298,22 +319,23 @@ void UIDialog::loadConfig()
   myDebuggerWidthLabel->setValue(w);
   myDebuggerHeightSlider->setValue(h);
   myDebuggerHeightLabel->setValue(h);
+#endif
 
   // UI palette
   const string& pal = instance().settings().getString("uipalette");
   myPalettePopup->setSelected(pal, "1");
 
   // Listwidget quick delay
-  int delay = instance().settings().getInt("listdelay");
-  if(delay < 300 || delay > 1000) delay = 300;
-  myListDelaySlider->setValue(delay);
-  myListDelayLabel->setValue(delay);
+  const string& delay = instance().settings().getString("listdelay");
+  myListDelayPopup->setSelected(delay, "300");
 
   // Mouse wheel lines
-  int mw = instance().settings().getInt("mwheel");
-  if(mw < 1 || mw > 10) mw = 1;
-  myWheelLinesSlider->setValue(mw);
-  myWheelLinesLabel->setValue(mw);
+  const string& mw = instance().settings().getString("mwheel");
+  myWheelLinesPopup->setSelected(mw, "1");
+
+  // Showinfo
+  const string& si = instance().settings().getString("showinfo");
+  myShowInfoPopup->setSelected(si, "1");
 
   myTab->loadConfig();
 }
@@ -333,9 +355,6 @@ void UIDialog::saveConfig()
   instance().settings().setString("romviewer",
     myRomViewerPopup->getSelectedTag());
 
-  // Use ROM launcher
-  instance().settings().setBool("uselauncher", myUseLauncherBox->getState());
-
   // Debugger size
   instance().settings().setSize("debuggerres", 
     myDebuggerWidthSlider->getValue(), myDebuggerHeightSlider->getValue());
@@ -345,14 +364,18 @@ void UIDialog::saveConfig()
     myPalettePopup->getSelectedTag());
 
   // Listwidget quick delay
-  int delay = myListDelaySlider->getValue();
-  instance().settings().setInt("listdelay", delay);
-  ListWidget::setQuickSelectDelay(delay);
+  instance().settings().setString("listdelay",
+    myListDelayPopup->getSelectedTag());
+  ListWidget::setQuickSelectDelay(atoi(myListDelayPopup->getSelectedTag().c_str()));
 
   // Mouse wheel lines
-  int mw = myWheelLinesSlider->getValue();
-  instance().settings().setInt("mwheel", mw);
-  ScrollBarWidget::setWheelLines(mw);
+  instance().settings().setString("mwheel",
+    myWheelLinesPopup->getSelectedTag());
+  ScrollBarWidget::setWheelLines(atoi(myWheelLinesPopup->getSelectedTag().c_str()));
+
+  // Show info
+  instance().settings().setString("showinfo",
+    myShowInfoPopup->getSelectedTag());
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -370,7 +393,6 @@ void UIDialog::setDefaults()
       myLauncherHeightLabel->setValue(h);
       myLauncherFontPopup->setSelected("medium", "");
       myRomViewerPopup->setSelected("0", "");
-      myUseLauncherBox->setState(true);
       break;
     }
 
@@ -383,10 +405,9 @@ void UIDialog::setDefaults()
 
     case 2:  // Misc. options
       myPalettePopup->setSelected("1", "1");
-      myListDelaySlider->setValue(300);
-      myListDelayLabel->setValue(300);
-      myWheelLinesSlider->setValue(4);
-      myWheelLinesLabel->setValue(4);
+      myListDelayPopup->setSelected("300", "300");
+      myWheelLinesPopup->setSelected("4", "4");
+      myShowInfoPopup->setSelected("1", "1");
       break;
 
     default:
@@ -417,14 +438,6 @@ void UIDialog::handleCommand(CommandSender* sender, int cmd, int data, int id)
       myDebuggerHeightLabel->setValue(myDebuggerHeightSlider->getValue());
       break;
 
-    case kLQDelayChanged:
-      myListDelayLabel->setValue(myListDelaySlider->getValue());
-      break;
-
-    case kWLinesChanged:
-      myWheelLinesLabel->setValue(myWheelLinesSlider->getValue());
-      break;
-
     case kOKCmd:
       saveConfig();
       close();
