@@ -8,12 +8,13 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2009 by Bradford W. Mott and the Stella team
+// Copyright (c) 1995-2010 by Bradford W. Mott, Stephen Anthony
+// and the Stella Team
 //
-// See the file "license" for information on usage and redistribution of
+// See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: SettingsWin32.cxx 1724 2009-05-13 13:55:40Z stephena $
+// $Id: SettingsWin32.cxx 2059 2010-07-06 15:08:35Z stephena $
 //============================================================================
 
 #include "bspf.hxx"
@@ -25,10 +26,7 @@ SettingsWin32::SettingsWin32(OSystem* osystem)
   : Settings(osystem)
 {
   setInternal("gl_lib", "opengl32.dll");
-  // Anything less than this usually causes sound skipping
-  setInternal("fragsize", "2048");
-  // Most Windows systems work better without this
-  setInternal("romdir", "c:\\");
+  setInternal("fragsize", "512");
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
